#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.entities.autoshare.model import Autoshare
from labstep.generic.entity.model import Entity
from labstep.entities.workspaceMember.model import WorkspaceMember
from labstep.entities.sharelink.model import Sharelink
from labstep.service.helpers import getTime


class Workspace(Entity):
    """
    Represents a Labstep Workspace.

    To see all attributes of the workspace run
    ::
        print(my_workspace)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(my_workspace.name)
        print(my_workspace.id)
    """

    __entityName__ = "group"

    def edit(self, name=None, extraParams={}):
        """
        Edit an existing Workspace.

        Parameters
        ----------
        name (str)
            The new name of the Workspace.

        Returns
        -------
        :class:`~labstep.entities.workspace.model.Workspace`
            An object representing the edited Workspace.

        Example
        -------
        ::

            my_workspace = user.getWorkspace(17000)
            my_workspace.edit(name='A New Workspace Name')
        """
        import labstep.entities.workspace.repository as workspaceRepository

        return workspaceRepository.editWorkspace(self, name, extraParams=extraParams)

    def delete(self):
        """
        Delete an existing Workspace.

        Example
        -------
        ::

            my_workspace = user.getWorkspace(17000)
            my_workspace.delete()
        """
        import labstep.entities.workspace.repository as workspaceRepository

        return workspaceRepository.editWorkspace(self, deleted_at=getTime())

    # getMany()
    def getExperiments(
        self,
        count=100,
        search_query=None,
        created_at_from=None,
        created_at_to=None,
        tag_id=None,
        collection_id=None,
        extraParams={},
    ):
        """
        Retrieve a list of Experiments within this specific Workspace,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of Experiments to retrieve.
        search_query (str)
            Search for Experiments containing this string in the name or entry.
        created_at_from (str)
            The start date of the search range, must be
            in the format of 'YYYY-MM-DD'.
        created_at_to (str)
            The end date of the search range, must be
            in the format of 'YYYY-MM-DD'.
        tag_id (int)
            The id of a tag to filter by.
        collection_id (int)
            Get experiments in this collection.

        Returns
        -------
        List[:class:`~labstep.entities.experiment.model.Experiment`]
            A list of Labstep Experiments.

        Example
        -------
        ::

            entity = workspace.getExperiments(search_query='bacteria',
                                              created_at_from='2019-01-01',
                                              created_at_to='2019-01-31',
                                              tag_id=800)
        """
        import labstep.entities.experiment.repository as experimentRepository

        extraParams = {"group_id": self.id, **extraParams}

        return experimentRepository.getExperiments(
            self.__user__,
            count=count,
            search_query=search_query,
            created_at_from=created_at_from,
            created_at_to=created_at_to,
            tag_id=tag_id,
            collection_id=collection_id,
            extraParams=extraParams,
        )

    def getProtocols(
        self,
        count=100,
        search_query=None,
        created_at_from=None,
        created_at_to=None,
        tag_id=None,
        collection_id=None,
        extraParams={},
    ):
        """
        Retrieve a list of Protocols within this specific Workspace,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of Protocols to retrieve.
        search_query (str)
            Search for Protocols with this 'name'.
        created_at_from (str)
            The start date of the search range, must be
            in the format of 'YYYY-MM-DD'.
        created_at_to (str)
            The end date of the search range, must be
            in the format of 'YYYY-MM-DD'.
        tag_id (int)
            The id of a tag to filter by.
        collection_id (int)
            Get protocols in this collection.

        Returns
        -------
        List[:class:`~labstep.entities.protocol.model.Protocol`]
            A list of Labstep Protocols.

        Example
        -------
        ::

            entity = workspace.getProtocols(search_query='bacteria',
                                            created_at_from='2019-01-01',
                                            created_at_to='2019-01-31',
                                            tag_id=800)
        """
        import labstep.entities.protocol.repository as protocolRepository

        extraParams = {"group_id": self.id, **extraParams}

        return protocolRepository.getProtocols(
            self.__user__,
            count=count,
            search_query=search_query,
            created_at_from=created_at_from,
            created_at_to=created_at_to,
            tag_id=tag_id,
            collection_id=collection_id,
            extraParams=extraParams,
        )

    def getResources(self, count=100, search_query=None, tag_id=None, extraParams={}):
        """
        Retrieve a list of Resources within this specific Workspace,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of Resources to retrieve.
        search_query (str)
            Search for Resources with this 'name'.
        tag_id (int)
            The id of a tag to filter by.

        Returns
        -------
        List[:class:`~labstep.entities.resource.model.Resource`]
            A list of Labstep Resources.

        Example
        -------
        ::

            entity = workspace.getResources(search_query='bacteria',
                                            tag_id=800)
        """
        import labstep.entities.resource.repository as resourceRepository

        extraParams = {"group_id": self.id, **extraParams}

        return resourceRepository.getResources(
            self.__user__, count, search_query, tag_id, extraParams=extraParams
        )

    def getResourceCategorys(
        self, count=100, search_query=None, tag_id=None, extraParams={}
    ):
        """
        Retrieve a list of Resource Categories within this specific Workspace,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of ResourceCategorys to retrieve.
        search_query (str)
            Search for ResourceCategorys with this 'name'.
        tag_id (int)
            The id of a tag to filter by.

        Returns
        -------
        List[:class:`~labstep.entities.resourceCategory.model.ResourceCategory`]
            A list of Labstep ResourceCategorys.

        Example
        -------
        ::

            entity = workspace.getResourceCategorys(search_query='properties',
                                                    tag_id=800)
        """
        import labstep.entities.resourceCategory.repository as resourceCategoryRepository

        extraParams = {"group_id": self.id, **extraParams}

        return resourceCategoryRepository.getResourceCategorys(
            self.__user__, count, search_query, tag_id, extraParams=extraParams
        )

    def getResourceLocations(self, count=100, search_query=None, extraParams={}):
        """
        Retrieve a list of Resource Locations within this specific Workspace,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of ResourceLocations to retrieve.
        search_query (str)
            Search for ResourceLocations with this 'name'.

        Returns
        -------
        List[:class:`~labstep.entities.resourceLocation.model.ResourceLocation`]
            A list of ResourceLocation objects.

        Example
        -------
        ::

            entity = workspace.getResourceLocations(search_query='properties',
                                                    tag_id=800)
        """
        import labstep.entities.resourceLocation.repository as resourceLocationRepository

        extraParams = {"group_id": self.id, **extraParams}

        return resourceLocationRepository.getResourceLocations(
            self.__user__, count, search_query, extraParams=extraParams
        )

    def getOrderRequests(
        self, count=100, name=None, status=None, tag_id=None, extraParams={}
    ):
        """
        Retrieve a list of Order Requests within this specific Workspace,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of OrderRequests to retrieve.
        name (str)
            Search for OrderRequests with this 'name'.

        Returns
        -------

        List[:class:`~labstep.entities.orderRequest.model.OrderRequest`]
            A list of Labstep OrderRequests.

        Example
        -------
        ::

            entity = workspace.getOrderRequests(name='polymerase')
        """
        import labstep.entities.orderRequest.repository as orderRequestRepository

        extraParams = {"group_id": self.id, **extraParams}

        return orderRequestRepository.getOrderRequests(
            self.__user__,
            count=count,
            search_query=name,
            status=status,
            tag_id=tag_id,
            extraParams=extraParams,
        )

    def getTags(self, count=1000, search_query=None, type=None, extraParams={}):
        """
        Retrieve a list of Tags within this specific Workspace,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of Tags to retrieve.
        type (str)
            Return only tags of a certain type. Options are:
            'experiment_workflow', 'protocol_collection',
            'resource', 'order_request'.
        search_query (str)
            Search for Tags with this 'name'.

        Returns
        -------
        List[:class:`~labstep.entities.tag.model.Tag`]
            A list of Labstep Tags.

        Example
        -------
        ::

            entity = workspace.getTags(search_query='bacteria')
        """
        import labstep.entities.tag.repository as tagRepository

        extraParams = {"group_id": self.id, **extraParams}

        return tagRepository.getTags(
            self.__user__, count, type, search_query, extraParams=extraParams
        )

    def addMember(self, user_id):
        """
        Add a new member to the workspace.

        Note: can only be used to add users from within the same Organization.

        Parameters
        ----------
        user_id (int)
            The id of the user to add as a member

        Returns
        -------
        :class:`~labstep.entities.workspaceMember.model.WorkspaceMember`
            An object representing the member of the workspace and their permissions within the workspace.

        Example
        -------
        ::

            newMember = workspace.addMember(user_id=123)
        """
        import labstep.entities.workspaceMember.repository as workspaceMemberRepository
        return workspaceMemberRepository.addMember(self.__user__, workspace_id=self.id, user_id=user_id)

    def getMembers(self, count=100, search_query=None, extraParams={}):
        """
        Retrieve a list of the members of the workspace.

        Parameters
        ----------
        count (int)
            The number of members to retrieve.

        search_query (str)
            Search for members by name.

        Returns
        -------
        List[:class:`~labstep.entities.workspaceMember.model.WorkspaceMember`]
            A list of the members of the workspace and their permissions.

        Example
        -------
        ::

            members = workspace.getMembers(search_query='john')
        """
        import labstep.entities.workspaceMember.repository as workspaceMemberRepository
        return workspaceMemberRepository.getMembers(self.__user__, workspace_id=self.id, search_query=search_query, extraParams=extraParams)

    def getFiles(self, count=100, search_query=None, file_type=None, extraParams={}):
        """
        Retrieve a list of Files in the Workspace on Labstep,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of files to retrieve.
        file_type (str)
            Return only files of a certain type. Options are:
            'csv', 'doc',
            'docx', 'jpg', 'pdf','png','ppt','pptx','svg','xls','xlsx','xml' or 'generic' for all others.
        search_query (str)
            Search for files with this name.

        Returns
        -------
        List[:class:`~labstep.entities.file.model.File`]
            A list of Labstep Files.

        Example
        -------
        ::

            files = workspace.getFiles(search_query='bacteria')
        """
        import labstep.entities.file.repository as fileRepository

        extraParams = {"group_id": self.id, **extraParams}

        return fileRepository.getFiles(
            self.__user__, count, search_query, file_type, extraParams=extraParams
        )

    def getDevices(self, count=100, search_query=None, extraParams={}):
        """
        Retrieve a list of Devices within this specific Workspace,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of Devices to retrieve.
        search_query (str)
            Search for Devices by name of metadata fields.

        Returns
        -------
        List[:class:`~labstep.entities.device.model.Device`]
            A list of Labstep Devices.

        Example
        -------
        ::

            devices = workspace.getDevices(search_query='microscope')
        """
        import labstep.entities.device.repository as deviceRepository

        extraParams = {"group_id": self.id, **extraParams}

        return deviceRepository.getDevices(
            self.__user__, count, search_query, extraParams=extraParams
        )

    def sendInvites(self, emails, message):
        """
        Send invites to a Labstep Workspace via email.

        Parameters
        ----------
        emails (list)
            A list of the emails to send the invite to.
        message (str)
            A message to send with the invite.

        Returns
        -------
        None

        Example
        -------
        ::

            workspace.sendInvites(
                emails=['collegue1@labstep.com','collegue2@labstep.com'],
                message='Hi, please collaborate with me on Labstep!')
        """
        self.getSharelink().sendEmails(emails=emails, message=message)

    def getSharelink(self):
        """
        Get the sharelink for the workspace.

        Returns
        -------
        :class:`~labstep.entities.sharelink.model.Sharelink`
            The sharelink for the workspace

        """

        import labstep.entities.sharelink.repository as shareLinkRepository
        return shareLinkRepository.getSharelink(self)

    def getCollections(
        self, count=1000, search_query=None, type="experiment", extraParams={}
    ):
        """
        Retrieve a list of Collections within this specific Workspace,
        which can be filtered using the parameters:

        Parameters
        ----------
        count (int)
            The number of Collection to retrieve.
        type (str)
            Return only Collections of a certain type. Options are:
           'experiment', 'protocol'.
        search_query (str)
            Search for Collections with this 'name'.

        Returns
        -------
        List[:class:`~labstep.entities.collection.model.Collection`]
            A list of Labstep Collections.

        Example
        -------
        ::

            entity = workspace.getCollections(search_query='bacteria')
        """
        extraParams = {"group_id": self.id, **extraParams}
        import labstep.entities.collection.repository as collectionRepository

        return collectionRepository.getCollections(
            self.__user__, count, type, search_query, extraParams=extraParams
        )

    def newCollection(self, name, type="experiment"):
        """
        Create a new Collection within the Workspace for Experiments or Protocols.

        Parameters
        ----------
        user (obj)
            The Labstep user creating the Collection.
            Must have property 'api_key'. See 'login'.
        name (str)
            Name of the new Collection.
        type (str)
            Return only collections of a certain type. Options are:
           'experiment', 'protocol'. Defaults to 'experiment'

        Returns
        -------
        collection
            An object representing the new Labstep Collection.
        """
        import labstep.entities.collection.repository as collectionRepository

        return collectionRepository.newCollection(
            self.__user__, name=name, type=type, extraParams={
                "group_id": self.id}
        )

    def setHome(self):
        """
        Sets this workspace as the default workspace for the active user.
        """
        member = WorkspaceMember(self.logged_user_user_group, self.__user__)
        import labstep.generic.entity.repository as entityRepository

        return entityRepository.editEntity(member, {"is_home": True})

    def setAutosharing(
        self, experiment_sharing=None, protocol_sharing=None, resource_sharing=None
    ):
        """
        Parameters
        ----------
        experiment_sharing (str)
            Automatically share experiments
            you create and own with this workspace. Set to True or False

        protocol_sharing (str)
            Automatically share protocols
            you create and own with this workspace. Set to True or False

        resource_sharing (str)
            Automatically share resources
            you create and own with this workspace. Set to True or False

        Returns
        -------
        :class:`~labstep.entities.autoshare.model.Autoshare`
            An object representing the Autosharing policy.

        Example
        -------
        ::

            # Get an workspace
            workspace = user.getWorkspaces(123)

            workspace.setAutosharing(experiment_sharing='view')
        """
        self.update()

        if self.security_policy is None:
            import labstep.generic.entity.repository as entityRepository

            policy = entityRepository.newEntity(
                self.__user__,
                Autoshare,
                {"user_group_id": self.logged_user_user_group["id"]},
            )
        else:
            policy = Autoshare(self.security_policy, self.__user__)

        return policy.edit(
            experiment_sharing=experiment_sharing,
            protocol_sharing=protocol_sharing,
            resource_sharing=resource_sharing,
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>
from deprecated import deprecated
from labstep.entities.protocolDataField.model import ProtocolDataField
from labstep.generic.primaryEntity.model import PrimaryEntity
from labstep.entities.protocolStep.model import ProtocolStep
from labstep.entities.protocolTable.model import ProtocolTable
from labstep.entities.protocolTimer.model import ProtocolTimer
from labstep.entities.protocolVersion.model import ProtocolVersion
from labstep.entities.protocolMaterial.model import ProtocolMaterial
import labstep.entities.protocolMaterial.repository as protocolMaterialRepository
from labstep.generic.entityList.model import EntityList
from labstep.entities.file.model import File
import labstep.entities.file.repository as fileRepository
from labstep.service.helpers import getTime


class Protocol(PrimaryEntity):
    """
    Represents a Protocol on Labstep.

    To see all attributes of a protocol run
    ::
        print(my_protocol)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(my_protocol.name)
        print(my_protocol.id)
    """

    __entityName__ = "protocol-collection"
    __searchKey__ = 'label'

    def edit(self, name=None, body=None, extraParams={}):
        """
        Edit an existing Protocol.

        Parameters
        ----------
        name (str)
            The name of the Protocol.
        body (dict):
            JSON representing the the protocol document.

        Returns
        -------
        :class:`~labstep.entities.protocol.model.Protocol`
            An object representing the edited Protocol.

        Example
        -------
        ::

            my_protocol = user.getProtocol(17000)
            my_protocol.edit(name='A New Protocol Name')
        """
        import labstep.entities.protocol.repository as protocolRepository

        return protocolRepository.editProtocol(
            self, name=name, body=body, extraParams=extraParams
        )

    def delete(self):
        """
        Delete an existing Protocol.

        Example
        -------
        ::

            my_protocol = user.getProtocol(17000)
            my_protocol.delete()
        """
        import labstep.entities.protocol.repository as protocolRepository

        return protocolRepository.editProtocol(self, deleted_at=getTime())

    def newVersion(self):
        """
        Start a new version of the Protocol.

        Example
        -------
        ::

            my_protocol = user.getProtocol(17000)
            new_version = my_protocol.newVersion()
        """
        import labstep.generic.entity.repository as entityRepository

        entityRepository.newEntity(
            self.__user__, ProtocolVersion, {"collection_id": self.id}
        )
        return self.update()

    def getBody(self):
        """
        Returns the body of the protocol as a JSON document

        Example
        -------
        ::

            my_protocol = user.newProtocol('My API Protocol')

            my_protocol.edit(body={
                "type": "doc",
                "content": [
                    {
                        "type": "paragraph",
                        "attrs": {"align": None},
                        "content": [
                            {
                                "type": "text",
                                "text": "This is the the body of my protocol"
                            }
                        ]
                    },
                    {
                        "type": "paragraph",
                        "attrs": {"align": None}
                    }
                ]
            })

            my_protocol.getBody()
        """
        self.update()
        if "state" not in self.last_version:
            return None
        return self.last_version["state"]

    def addSteps(self, N):
        """
        Add steps to the protocol

        Parameters
        ----------
        N (int)
            The number of steps to add.

        Example
        -------
        ::

            my_protocol = user.newProtocol('My API Protocol')
            my_protocol.addSteps(5)
        """
        import labstep.generic.entity.repository as entityRepository

        steps = [{"protocol_id": self.last_version["id"]}] * N
        return entityRepository.newEntities(self.__user__, ProtocolStep, steps)

    def getSteps(self):
        """
        Returns a list of the steps in a Protocol.

        Returns
        -------
        List[:class:`~labstep.entities.protocolStep.model.ProtocolStep`]
            List of the steps in Protocol.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            protocol_steps = protocol.getSteps()
            protocol_steps[0].attributes()
        """
        self.update()
        if "protocol_steps" not in self.last_version:
            return []
        steps = self.last_version["protocol_steps"]
        return EntityList(steps, ProtocolStep, self.__user__)

    def getDataFields(self):
        """
        Retrieve the Data Fields of a Protocol.

        Returns
        -------
        :class:`~labstep.entities.metadata.model.Metadata`
            An array of objects representing the Labstep Data Fields
            on a Protocol.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            metadata = protocol.getDataFields()
        """
        self.update()
        if "metadatas" not in self.last_version["metadata_thread"]:
            return []

        def addId(field):
            field['protocol_id'] = self.id
            return field

        return EntityList(map(addId, self.last_version["metadata_thread"]["metadatas"]), ProtocolDataField, self.__user__)

    def addDataField(
        self,
        fieldName,
        fieldType="default",
        value=None,
        date=None,
        number=None,
        unit=None,
        filepath=None,
        extraParams={},
    ):
        """
        Add a Data Field to a Protocol.

        Parameters
        ----------
        fieldName (str)
            The name of the field.
        fieldType (str)
            The field type. Options are: "default", "date",
            "quantity", or "number". The "default" type is "Text".
        value (str)
            The value accompanying the fieldName entry.
        date (str)
            The date and time accompanying the fieldName entry. Must be
            in the format of "YYYY-MM-DD HH:MM".
        number (float)
            The quantity.
        unit (str)
            The unit accompanying the number entry.

        Returns
        -------
        :class:`~labstep.entities.metadata.model.Metadata`
            An object representing the new Labstep Data Field.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            dataField = protocol.addDataField("Refractive Index",
                                               value="1.73")
        """
        import labstep.entities.protocolDataField.repository as protocolDataFieldRepository

        return protocolDataFieldRepository.addDataFieldTo(
            ProtocolVersion(self.last_version, self.__user__),
            fieldName=fieldName,
            fieldType=fieldType,
            value=value,
            date=date,
            number=number,
            unit=unit,
            filepath=filepath,
            extraParams=extraParams,
        )

    def addMaterial(
        self, name=None, amount=None, units=None, resource_id=None, extraParams={}
    ):
        """
        Add a new material to the Protocol.

        Parameters
        ----------
        name (str)
            The name of the material to add.
        amount (str)
            The amount required by the protocol.
        units (str)
            The units for the amount.
        resource_id (int)
            The id of the :class:`~labstep.entities.resource.model.Resource` recommended for
            use with the protocol.

        Returns
        -------
        :class:`~labstep.entities.protocolMaterial.model.ProtocolMaterial`
            The newly added material entity.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            resource = user.getResources(search_query='Sample A')[0]
            protocol.addMaterial(name='Sample A', amount='2', units='ml',
                                 resource_id=resource.id)
        """
        return protocolMaterialRepository.newProtocolMaterial(self.__user__,
                                                              protocol_id=self.last_version["id"],
                                                              resource_id=resource_id, name=name,
                                                              amount=amount,
                                                              units=units,
                                                              extraParams=extraParams)

    def getMaterials(self, count=100, extraParams={}):
        """
        Returns a list of the materials in a Protocol.

        Returns
        -------
        List[:class:`~labstep.entities.protocolMaterial.model.ProtocolMaterial`]
            List of the materials in a Protocol.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            protocol_materials = protocol.getMaterials()
            protocol_materials[0].attributes()
        """
        self.update()
        if "protocol_values" not in self.last_version:
            return []
        materials = self.last_version["protocol_values"]
        return EntityList(materials, ProtocolMaterial, self.__user__)

    def addTimer(self, name=None, hours=None, minutes=None, seconds=None):
        """
        Add a new timer to the Protocol.

        Parameters
        ----------
        name (str)
            The name of the timer.
        hours (int)
            The hours of the timer.
        minutes (int)
            The minutes of the timer.
        seconds (int)
            The seconds of the timer.

        Returns
        -------
        :class:`~labstep.entities.protocolTimer.model.ProtocolTimer`
            The newly added timer entity.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            protocol.addTimer(name='Refluxing', hours='4', minutes='30')
        """
        import labstep.generic.entity.repository as entityRepository

        params = {
            "protocol_id": self.last_version["id"],
            "name": name,
            "hours": hours,
            "minutes": minutes,
            "seconds": seconds,
        }
        return entityRepository.newEntity(self.__user__, ProtocolTimer, params)

    def getTimers(self):
        """
        Returns a list of the timers in a Protocol.

        Returns
        -------
        List[:class:`~labstep.entities.protocolTimer.model.ProtocolTimer`]
            List of the timers in a Protocol.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            protocol_timers = protocol.getTimers()
            protocol_timers[0].attributes()
        """
        self.update()
        if "protocol_timers" not in self.last_version:
            return []

        timers = self.last_version["protocol_timers"]
        return EntityList(timers, ProtocolTimer, self.__user__)

    def addTable(self, name=None, data=None):
        """
        Add a new table to the Protocol.

        Parameters
        ----------
        name (str)
            The name of the table.
        data (json)
            The data of the table in json format.

        Returns
        -------
        :class:`~labstep.entities.protocolTable.model.ProtocolTable`
            The newly added table entity.

        Example
        -------
        ::

            data = {
                "rowCount": 12,
                "columnCount": 12,
                "colHeaderData": {},
                "data": {
                    "dataTable": {
                        0: {
                            0: {
                                "value": 'Cell A1'
                            },
                            1: {
                                "value": 'Cell B1'
                            }
                        }
                    }
                }
            }

            protocol = user.getProtocol(17000)
            protocol.addTable(name='Calibration', data=data)
        """
        import labstep.generic.entity.repository as entityRepository

        params = {
            "protocol_id": self.last_version["id"], "name": name, "data": data}
        return entityRepository.newEntity(self.__user__, ProtocolTable, params)

    def getTables(self):
        """
        Returns a list of the tables in a Protocol.

        Returns
        -------
        List[:class:`~labstep.entities.protocolTable.model.ProtocolTable`]
            List of the tables in a Protocol.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            protocol_tables = protocol.getTables()
            protocol_tables[0].attributes()
        """
        self.update()
        if "protocol_tables" not in self.last_version:
            return []

        tables = self.last_version["protocol_tables"]
        return EntityList(tables, ProtocolTable, self.__user__)

    def addToCollection(self, collection_id):
        """
        Add the protocol to a collection.

        Parameters
        ----------
        collection_id (int)
            The id of the collection to add to

        Returns
        -------
        None
        """
        import labstep.entities.collection.repository as collectionRepository

        return collectionRepository.addToCollection(self, collection_id=collection_id)

    def getCollections(self):
        """
        Returns the list of collections the protocol is in.
        """
        import labstep.entities.collection.repository as collectionRepository

        return collectionRepository.getAttachedCollections(self)

    def removeFromCollection(self, collection_id):
        """
        Remove the protocol from a collection.

        Parameters
        ----------
        collection_id (int)
            The id of the collection to remove from

        """
        import labstep.entities.collection.repository as collectionRepository

        return collectionRepository.removeFromCollection(self, collection_id)

    def addFile(self, filepath=None, rawData=None):
        """
        Add a file to a Protocol.

        Parameters
        ----------
        filepath (str)
            The path to the file to upload.

        Returns
        -------
        :class:`~labstep.file.File`
            The newly added file entity.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            protocol.addFile(filepath='./my_file.csv')
        """
        params = {'protocol_id': self.last_version['id']}
        return fileRepository.newFile(self.__user__,
                                      filepath=filepath,
                                      rawData=rawData,
                                      extraParams=params)

    def getFiles(self):
        """
        Returns a list of the files in a Protocol.

        Returns
        -------
        List[:class:`~labstep.file.File`]
            List of the files in a Protocol.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            protocol_files = protocol.getFiles()
        """
        self.update()
        if 'files' not in self.last_version:
            return []

        files = self.last_version['files']
        return EntityList(files, File, self.__user__)

    def export(self, path):
        """
        Export the protocol to the directory specified.

        Parameters
        -------
        path (str)
            The path to the directory to save the protocol.

        Example
        -------
        ::

            experiment = user.getProtocol(17000)
            experiment.export('/my_folder')
        """
        import labstep.entities.protocol.repository as protocolRepository

        return protocolRepository.exportProtocol(self, path)

    @ deprecated(version='3.3.2', reason="You should use experiment.addDataField instead")
    def addDataElement(self, *args, **kwargs):
        return self.addDataField(*args, **kwargs)

    @ deprecated(version='3.3.2', reason="You should use experiment.getDataFields instead")
    def getDataElements(self):
        return self.getDataFields()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import os
import labstep.entities.user.repository as userRepository


def newUser(
    first_name, last_name, email, password, share_link_token=None, extraParams={}
):
    """
    TODO
    """
    return userRepository.newUser(
        first_name, last_name, email, password, share_link_token, extraParams
    )


def authenticate(username=None, apikey=None):
    """
    Returns an authenticated Labstep User object to allow
    you to interact with the Labstep API.

    Parameters
    ----------
    username (str)
        Your Labstep username.
    apikey (str)
        An apikey for the user.

    Returns
    -------
    :class:`~labstep.entities.user.model.User`
        An object representing a user on Labstep.

    Example
    -------
    ::

        import labstep

        user = labstep.authenticate('myaccount@labstep.com', 'MY_API_KEY')
    """
    if (apikey is None):
        apikey = os.environ['LABSTEP_API_KEY']

    return userRepository.authenticate(username, apikey)


def login(username, password):
    """
    Returns an authenticated Labstep User object to allow
    you to interact with the Labstep API.

    Parameters
    ----------
    username (str)
        Your Labstep username.
    password (str)
        Your Labstep password.

    Returns
    -------
    :class:`~labstep.entities.user.model.User`
        An object representing a user on Labstep.

    Example
    -------
    ::

        import labstep

        user = labstep.login('myaccount@labstep.com', 'mypassword')
    """
    return userRepository.login(username, password)


def impersonate(username, apikey):
    return userRepository.impersonate(username, apikey)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity
from labstep.service.helpers import getTime


class Comment(Entity):
    __entityName__ = "comment"

    def edit(self, body, extraParams={}):
        """
        Edit an existing comment/caption.

        Parameters
        ----------
        body (str)
            The body of the new comment.

        Returns
        -------
        :class:`~labstep.entities.comment.model.Comment`
            An object representing the edited comment.

        Example
        -------
        ::

            my_comment.edit(body='My new comment.')
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.editComment(self, body, extraParams=extraParams)

    def delete(self):
        """
        Delete the comment.

        Example
        -------
        ::

            comment.delete()
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.editComment(
            self, extraParams={"deleted_at": getTime()}
        )

    def addComment(self, body, filepath=None):
        """
        Add a comment and/or file about this comment.

        Parameters
        ----------
        body (str)
            The body of the comment.
        filepath (str)
            A Labstep File entity to attach to the comment,
            including the filepath.

        Returns
        -------
        :class:`~labstep.entities.comment.model.Comment`
            The comment added.

        Example
        -------
        ::

            my_experiment = user.getExperiment(17000)
            my_experiment.getComments()[0].addComment(body='I am commenting!',
                                     filepath='pwd/file_to_upload.dat')
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.addCommentWithFile(self, body, filepath)

    def getComments(self, count=100):
        """
        Retrieve the Comments attached to this Comment.

        Returns
        -------
        List[:class:`~labstep.entities.comment.model.Comment`]
            List of the comments attached.

        Example
        -------
        ::

            entity = user.getExperiment(17000)
            comments = entity.getComments()[0].getComments()
            comments[0]
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.getComments(self, count)

    def export(self, rootPath):
        import labstep.entities.comment.repository as commentRepository
        return commentRepository.exportComment(self, rootPath)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Thomas Bullier <thomas@labstep.com>

from labstep.generic.primaryEntity.model import PrimaryEntity


class JupyterNotebook(PrimaryEntity):
    """
    Represents an JupyterNotebook on Labstep.
    """

    __entityName__ = "jupyter-notebook"
    __hasGuid__ = True

    def edit(self, name=None, status=None, data=None, extraParams={}):
        """
        Edit an existing JupyterNotebook.

        Parameters
        ----------
        name (str)
            A display name.
        status (str)
            Server status (stopped, pending, started).
        data (str)
            JupyterNotebook ipyb JSON data.

        Returns
        -------
        :class:`~labstep.entities.jupyterNotebook.model.JupyterNotebook`
            An object representing the edited JupyterNotebook.

        Example
        -------
        ::

            my_jupyterNotebook = user.getJupyterNotebook("872b3e7e-e21f-4403-9ef3-3650fe0d86ba")
            my_jupyterNotebook.edit(data='{test: 42}')
        """
        import labstep.entities.jupyterNotebook.repository as jupyterNotebookRepository

        return jupyterNotebookRepository.editJupyterNotebook(
            self, name=name, status=status, data=data, extraParams=extraParams
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import labstep.entities.file.repository as fileRepository
import labstep.generic.entity.repository as entityRepository
from labstep.service.helpers import handleDate, handleString
from labstep.entities.metadata.model import Metadata, FIELDS, ALLOWED_FIELDS


def getMetadata(entity, count=1000, extraParams={}):
    if hasattr(entity, 'metadata_thread') is False:
        entity.update()
    params = {
        "metadata_thread_id": entity.metadata_thread["id"], **extraParams}
    return entityRepository.getEntities(
        entity.__user__, Metadata, count=count, filterParams=params
    )


def addMetadataTo(

    entity,
    fieldName,
    fieldType="default",
    value=None,
    date=None,
    number=None,
    unit=None,
    filepath=None,
    extraParams={},
):
    if filepath is not None:
        fileId = fileRepository.newFile(entity.__user__, filepath).id
    else:
        fileId = None

    if fieldType not in FIELDS:
        msg = "Not a supported metadata type '{}'".format(fieldType)
        raise ValueError(msg)

    allowedFieldsForType = set(ALLOWED_FIELDS[fieldType])
    fields = {
        "value": value,
        "date": date,
        "number": number,
        "unit": unit,
        "file_id": fileId,
    }
    fields = {k: v for k, v in fields.items() if v}
    fields = set(fields.keys())
    violations = fields - allowedFieldsForType
    if violations:
        msg = "Unallowed fields [{}] for type {}".format(
            ",".join(violations), fieldType
        )
        raise ValueError(msg)

    params = {
        "metadata_thread_id": entity.metadata_thread["id"],
        "type": fieldType,
        "label": handleString(fieldName),
        "value": handleString(value),
        "date": handleDate(date),
        "number": number,
        "unit": unit,
        "file_id": fileId,
        **extraParams,
    }

    return entityRepository.newEntity(entity.__user__, Metadata, params)


def editMetadata(metadata, fieldName=None, value=None, extraParams={}):
    params = {
        "label": handleString(fieldName),
        "value": handleString(value),
        **extraParams
    }
    return entityRepository.editEntity(metadata, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import labstep.entities.organizationUser.repository as organizationUserRepository
from labstep.generic.entity.model import Entity
import labstep.generic.entity.repository as entityRepository
import labstep.entities.workspace.repository as workspaceRepository
import labstep.entities.invitation.repository as invitationRepository


class Organization(Entity):
    __entityName__ = "organization"

    def edit(self, name, extraParams={}):
        """
        Edit Organization.

        Parameters
        ----------
        name (str)
            The name of the Organization.

        Returns
        -------
        :class:`~labstep.entities.organization.model.Organization`
            An object representing the organization.

        Example
        -------
        ::

            my_organization.edit(name='My new organization.')
        """
        import labstep.entities.organization.repository as organizationRepository

        return organizationRepository.editOrganization(self.__user__, name, extraParams=extraParams)

    def inviteUsers(self, emails, workspace_id=None):
        """
        Invite users to your organization.

        Parameters
        ----------
        emails (list)
            A list of email address to send invitations to.

        workspace_id (int)
            Optionally specifiy the id of a workspace to add the new users to.

        Example
        -------
        ::

            my_organization.inviteUsers(emails=['user1@labstep.com','user2@labstep.com'],
                workspace_id=123)
        """
        return invitationRepository.newInvitations(self.__user__,
                                                   invitationType='organization',
                                                   emails=emails,
                                                   organization_id=self.id,
                                                   workspace_id=workspace_id)

    def getWorkspaces(self, count=100, search_query=None):
        """
        Get the workspaces in your Organization

        Parameters
        ----------
        count (int)
            Number of workspaces to return.

        search_query (string)
            Search for specific workspaces by name.

        Returns
        -------
        List[:class:`~labstep.entities.workspace.model.Workspace`]
            A list of workspaces in the organization.

        Example
        -------
        ::

            workspaces = my_organization.getWorkspaces(search_query='R&D Workspace')
        """
        return workspaceRepository.getWorkspaces(self.__user__,
                                                 count=count,
                                                 search_query=search_query,
                                                 extraParams={'organization_id': self.id})

    def getUsers(self, count=100, extraParams={}):
        """
        Get the users in your Organization.

        Returns
        -------
        List[:class:`~labstep.entities.organizationUser.model.OrganizationUser`]
            A list of users in your organization

        Example
        -------
        ::

            users = my_organization.getUsers()
            user[0].disable()
        """
        return organizationUserRepository.getOrganizationUsers(self,
                                                               count=count,
                                                               extraParams=extraParams)

    def getPendingInvitations(self, extraParams={}):
        """
        Get pending invitations to your Organization.

        Returns
        -------
        List[:class:`~labstep.entities.invitation.model.Invitation`]
            A list of invitations sent

        Example
        -------
        ::

            invitations = my_organization.getPendingInvititations()
        """
        return invitationRepository.getInvitations(self.__user__,
                                                   self.id,
                                                   extraParams={'has_invited_user': False,
                                                                **extraParams})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity
from labstep.service.helpers import getTime


class ExperimentStep(Entity):
    __entityName__ = "experiment-step"

    def edit(self, completed_at=None):
        """
        Edit an existing Experiment Step.

        Parameters
        ----------
        completed_at (str)
            The datetime at which the Experiment Step was completed.

        Returns
        -------
        :class:`~labstep.entities.experimentStep.model.ExperimentStep`
            An object representing the edited Experiment Step.
        """
        import labstep.generic.entity.repository as entityRepository

        params = {"ended_at": completed_at}
        return entityRepository.editEntity(self, params)

    def complete(self):
        """
        Mark an existing Experiment Step as 'Complete'.

        Example
        -------
        ::

            experiment = user.getExperiment(17000)
            exp_protocol = experiment.getProtocols()[0]
            exp_protocol_steps = exp_protocol.getSteps()
            exp_protocol_steps[0].complete()
            exp_protocol_steps[1].complete()
            exp_protocol_steps[2].complete()
        """
        return self.edit(completed_at=getTime())

    def addComment(self, body, filepath=None):
        """
        Add a comment and/or file to this step.

        Parameters
        ----------
        body (str)
            The body of the comment.
        filepath (str)
            A Labstep File entity to attach to the comment,
            including the filepath.

        Returns
        -------
        :class:`~labstep.entities.comment.model.Comment`
            The comment added.

        Example
        -------
        ::

            experiment = user.getExperiment(17000)
            exp_protocol = experiment.getProtocols()[0]
            exp_protocol_steps = exp_protocol.getSteps()
            exp_protocol_steps[0].addComment('This step failed')
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.addCommentWithFile(self, body, filepath)

    def getComments(self, count=100):
        """
        Retrieve the Comments attached to this step.

        Parameters
        ----------

        count (int)
            The number of comments to return

        Returns
        -------
        List[:class:`~labstep.entities.comment.model.Comment`]
            List of the comments attached.

        Example
        -------
        ::

           experiment = user.getExperiment(17000)
            exp_protocol = experiment.getProtocols()[0]
            exp_protocol_steps = exp_protocol.getSteps()
            exp_protocol_steps[0].getComments()
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.getComments(self, count)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import inspect
import json
import pprint
from labstep.service.helpers import update


class Entity:

    def __init__(self, data, user):
        self.__user__ = user
        self.__data__ = data
        self.id = None
        update(self, data)

    def __repr__(self):
        all_attributes = inspect.getmembers(
            self, lambda a: not (inspect.isroutine(a)))
        entity_attributes = {
            k: v for k, v in all_attributes if not (k.startswith("__"))
        }
        pp = pprint.PrettyPrinter(indent=1)
        return pp.pformat(entity_attributes)

    def update(self):
        """
        Fetches the most up-to-date version of the entity from Labstep.
        """
        import labstep.generic.entity.repository as entityRepository

        data = entityRepository.getEntity(
            self.__user__, type(self), self.id).__data__
        self.__init__(data, self.__user__)
        return self

    def export(self, rootPath, folderName=None):
        """
        Export the entity to the directory specified. 

        Parameters
        -------
        path (str)
            The path to the directory to save the experiment.

        Example
        -------
        ::

            experiment = user.getExperiment(17000)
            experiment.export('/my_folder')
        """
        import labstep.generic.entity.repository as entityRepository
        return entityRepository.exportEntity(self, rootPath, folderName=folderName)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import json
from pathlib import Path
from pathvalidate import sanitize_filepath
from labstep.generic.entityList.model import EntityList
from labstep.service.config import configService
from labstep.service.helpers import (
    url_join,
    getHeaders,
)
from labstep.service.request import requestService
from labstep.config.export import entityNameInFolderName


def getLegacyEntity(user, entityClass, id):
    headers = getHeaders(user=user)
    url = url_join(configService.getHost(), "/api/generic/",
                   entityClass.__entityName__, str(id))
    response = requestService.get(url, headers=headers)
    return entityClass(json.loads(response.content), user)


def getEntity(user, entityClass, id, isDeleted="both", useGuid=False):
    if getattr(entityClass, "__isLegacy__", None):
        return getLegacyEntity(user, entityClass, id)

    identifier = 'guid' if getattr(
        entityClass, "__hasGuid__", None) or useGuid else 'id'

    params = {"is_deleted": isDeleted, "get_single": 1, identifier: id}

    headers = getHeaders(user=user)
    url = url_join(configService.getHost(), "/api/generic/",
                   entityClass.__entityName__)
    response = requestService.get(url, headers=headers, params=params)
    return entityClass(json.loads(response.content), user)


def getEntities(user, entityClass, count, filterParams={}):
    countParameter = min(count, 50) if count is not None else None
    searchParams = {"search": 1, "cursor": -1, "count": countParameter}

    params = {**searchParams, **filterParams}

    headers = getHeaders(user=user)
    url = url_join(configService.getHost(), "/api/generic/",
                   entityClass.__entityName__)
    response = requestService.get(url, params=params, headers=headers)
    resp = json.loads(response.content)
    items = resp["items"]
    expectedResults = min(
        resp["total"], count) if count is not None else resp["total"]
    while len(items) < expectedResults:
        params["cursor"] = resp["next_cursor"]
        response = requestService.get(url, headers=headers, params=params)
        resp = json.loads(response.content)
        items.extend(resp["items"])
    return EntityList(items, entityClass, user)


def newEntity(user, entityClass, fields):
    headers = getHeaders(user=user)
    url = url_join(configService.getHost(), "/api/generic/",
                   entityClass.__entityName__)
    fields = dict(
        filter(lambda field: field[1] is not None, fields.items()))

    if "group_id" not in fields and getattr(
        entityClass, "__hasParentGroup__", False
    ):
        fields["group_id"] = user.activeWorkspace

    response = requestService.post(url, headers=headers, json=fields)
    return entityClass(json.loads(response.content), user)


def linkEntities(user, entity1, entity2):
    headers = getHeaders(user)
    url = url_join(
        configService.getHost(),
        "api/generic/",
        entity1.__entityName__,
        str(entity1.id),
        entity2.__entityName__,
        str(entity2.id),
    )
    response = requestService.put(url, headers=headers)
    return json.loads(response.content)


def newEntities(user, entityClass, items):
    headers = getHeaders(user=user)
    url = url_join(configService.getHost(), "/api/generic/",
                   entityClass.__entityName__, "batch")
    response = requestService.post(
        url, headers=headers, json={"items": items, "group_id": user.activeWorkspace})
    entities = json.loads(response.content)
    return list(map(lambda entity: entityClass(entity, user), entities))


def editEntity(entity, fields):
    # Filter the 'fields' dictionary by removing {'fields': None}
    # to preserve the existing data in the 'fields', otherwise
    # the 'fields' will be overwritten to 'None'.
    identifier = entity.guid if getattr(
        entity, "__hasGuid__", None) and hasattr(entity, 'guid') else entity.id

    newFields = dict(
        filter(lambda field: field[1] is not None, fields.items()))
    headers = getHeaders(entity.__user__)
    url = url_join(configService.getHost(), "/api/generic/",
                   entity.__entityName__, str(identifier))
    response = requestService.put(url, json=newFields, headers=headers)
    entity.__init__(json.loads(response.content), entity.__user__)
    return entity


def exportEntity(entity, rootPath, folderName=None):

    from labstep.entities.file.model import File

    if folderName is None:
        if entityNameInFolderName:
            folderName = f'{entity.id} - {entity.name}' if hasattr(
                entity, 'name') else f'{entity.id}'
        else:
            folderName = str(entity.id)

    entityDir = Path(rootPath).joinpath(sanitize_filepath(folderName))
    entityDir.mkdir(parents=True, exist_ok=True)
    infoFile = entityDir.joinpath('entity.json')

    with open(infoFile, 'w') as out:
        json.dump(entity.__data__, out, indent=2)

    if hasattr(entity, 'file') and entity.file is not None:
        lsFile = File(entity.file, entity.__user__)
        fileDir = entityDir.joinpath('files')
        lsFile.export(fileDir)

    if hasattr(entity, 'files') and entity.files is not None:
        for file in entity.files:
            lsFile = File(file, entity.__user__)
            fileDir = entityDir.joinpath('files')
            lsFile.export(fileDir)

    return entityDir



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class Autoshare(Entity):
    """
    Represents an Autosharing rule on Labstep.
    """

    __entityName__ = "security-policy"
    __isLegacy__ = True

    def edit(
        self,
        experiment_sharing=None,
        protocol_sharing=None,
        resource_sharing=None,
        extraParams={},
    ):
        """
        Edit an autosharing policy.

        Parameters
        ----------
        experiment_sharing (str)
            Automatically share experiments
            you create and own with this workspace. Set to True or False

        protocol_sharing (str)
            Automatically share protocols
            you create and own with this workspace. Set to True or False

        resource_sharing (str)
            Automatically share resources
            you create and own with this workspace. Set to True or False


        Returns
        -------
        :class:`~labstep.entities.autoshare.model.Autoshare`
            An object representing the Autosharing policy.

        Example
        -------
        ::

            # Get an workspace
            workspace = user.getExperiment(123)

            # Get the sharelink for the experiment
            sharelink = experiment.getSharelink()

            # Edit the sharelink
            sharelink.edit(type='view')
        """
        from labstep.generic.entity.repository import editEntity

        options = {True: "edit", False: "none", None: None}

        fields = {
            "experiment_workflow": options[experiment_sharing],
            "protocol_collection": options[protocol_sharing],
            "resource": options[resource_sharing],
            **extraParams,
        }
        return editEntity(self, fields=fields)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import json
import urllib.parse
from labstep.entities.user.model import User
from labstep.service.config import configService
from labstep.service.helpers import url_join
from labstep.service.request import requestService


def getUser(username, apikey):
    url = url_join(configService.getHost(), f"api/generic/user/{username}")
    response = requestService.get(url, headers={"apikey": apikey})
    user = json.loads(response.content)
    return User(user)


def newUser(
    first_name,
    last_name,
    email,
    password,
    share_link_token=None,
    extraParams={},
):
    url = url_join(configService.getHost(), "public-api/user")
    params = {
        "first_name": first_name,
        "last_name": last_name,
        "email": email,
        "password": password,
        "share_link_token": share_link_token,
        **extraParams,
    }

    params = dict(
        filter(lambda field: field[1] is not None, params.items()))

    response = requestService.post(url=url, json=params, headers=None)
    return User(json.loads(response.content))


def authenticate(username, apikey):
    url = url_join(configService.getHost(), "api/generic/user/info")
    response = requestService.get(url, headers={"apikey": apikey})
    user = json.loads(response.content)
    user["api_key"] = apikey
    return User(user)


def login(username, password):
    params = {"username": username, "password": password}
    url = url_join(configService.getHost(), "/public-api/user/login")
    response = requestService.post(url=url, json=params, headers={})
    return User(json.loads(response.content))


def impersonate(username, apikey):
    user = getUser(username, apikey)
    url = url_join(configService.getHost(), "api/generic/token/impersonate")
    response = requestService.post(
        url, json={'guid': user.guid}, headers={"apikey": apikey})
    token = json.loads(response.content)['uuid']
    user.api_key = token
    return user



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class WorkspaceMember(Entity):
    """
    Represents a member of a Labstep Workspace.

    To see all attributes of the workspace run
    ::
        print(member)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(member.name)
    """

    __entityName__ = "user-group"

    def remove(self):
        """
        Remove this member from the workspace (requires owner permission)
        """
        import labstep.entities.workspaceMember.repository as workspaceMemberRepository
        return workspaceMemberRepository.removeMember(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.primaryEntity.model import PrimaryEntity
from labstep.service.helpers import getTime


class OrderRequest(PrimaryEntity):
    """
    Represents an Order Request on Labstep.

    To see all attributes of the order request run
    ::
        print(my_order_request)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(my_order_request.name)
        print(my_order_request.id)
    """

    __entityName__ = "order-request"

    def edit(
        self,
        status=None,
        resource_id=None,
        quantity=None,
        price=None,
        currency=None,
        extraParams={},
    ):
        """
        Edit an existing OrderRequest.

        Parameters
        ----------
        status (str)
            The status of the OrderRequest. Options are: "new", "approved",
            "ordered", "back_ordered", "received", and "cancelled".
        resource_id (int)
            The id of the :class:`~labstep.entities.resource.model.Resource` being requested.
        quantity (int)
            The quantity of the OrderRequest.
        price (int)
            The price of the OrderRequest.
        currency (str)
            The currency of the price in the format of the 3-letter
            currency code by country. For example, "EUR" for Euro, "GBP" for
            British Pound Sterling, "USD" for US Dollar, etc.

        Returns
        -------
        :class:`~labstep.entities.orderRequest.model.OrderRequest`
            An object representing the edited OrderRequest.

        Example
        -------
        ::

            my_orderRequest = user.getOrderRequest(17000)
            my_orderRequest.edit(status="back_ordered", quantity=3,
                                 price=50, currency="GBP")
        """
        import labstep.entities.orderRequest.repository as orderRequestRepository

        return orderRequestRepository.editOrderRequest(
            self,
            status=status,
            resource_id=resource_id,
            quantity=quantity,
            price=price,
            currency=currency,
            extraParams=extraParams,
        )

    def delete(self):
        """
        Delete an existing OrderRequest.

        Example
        -------
        ::

            my_orderRequest = user.getOrderRequest(17000)
            my_orderRequest.delete()
        """
        import labstep.entities.orderRequest.repository as orderRequestRepository

        return orderRequestRepository.editOrderRequest(self, deleted_at=getTime())

    def getResource(self):
        """
        Retrieve the Resource of the OrderRequest.

        Returns
        -------
        :class:`~labstep.entities.resource.model.Resource`
            An object representing the Resource of the OrderRequest.

        Example
        -------
        ::

            my_orderRequest = user.getOrderRequest(17000)
            my_orderRequest.getResource()
        """
        return self.__user__.getResource(self.resource["id"])

    def addMetadata(
        self,
        fieldName,
        fieldType="default",
        value=None,
        date=None,
        number=None,
        unit=None,
        filepath=None,
        extraParams={},
    ):
        """
        Add Metadata to an Order Request.

        Parameters
        ----------
        fieldName (str)
            The name of the field.
        fieldType (str)
            The Metadata field type. Options are: "default", "date",
            "numeric", or "file". The "default" type is "Text".
        value (str)
            The value accompanying the fieldName entry.
        date (str)
            The date accompanying the fieldName entry. Must be
            in the format of "YYYY-MM-DD HH:MM".
        number (float)
            The numeric value.
        unit (str)
            The unit accompanying the number entry.
        filepath (str)
            Local path to the file to upload for type 'file'

        Returns
        -------
        :class:`~labstep.entities.metadata.model.Metadata`
            An object representing the new Labstep Metadata.

        Example
        -------
        ::

            my_resource_category = user.getResourceCategory(17000)
            metadata = my_resource_category.addMetadata("Refractive Index",
                                               value="1.73")
        """
        import labstep.entities.metadata.repository as metadataRepository

        return metadataRepository.addMetadataTo(
            self,
            fieldName,
            fieldType,
            value,
            date,
            number,
            unit,
            filepath=filepath,
            extraParams=extraParams,
        )

    def getMetadata(self):
        """
        Retrieve the Metadata of a Labstep OrderRequest.

        Returns
        -------
        :class:`~labstep.entities.metadata.model.Metadata`
            An array of Metadata objects for the OrderRequest.

        Example
        -------
        ::

            my_order_request = user.getOrderRequest(17000)
            metadatas = my_order_request.getMetadata()
            metadatas[0].attributes()
        """
        import labstep.entities.metadata.repository as metadataRepository

        return metadataRepository.getMetadata(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.service.config import configService
from labstep.service.helpers import getHeaders, url_join
from labstep.entities.organization.model import Organization
import labstep.generic.entity.repository as entityRepository
from labstep.service.request import requestService


def getOrganization(user, id, extraParams={}):
    return entityRepository.getEntity(user, Organization, id, extraParams)


def editOrganization(organization, name, extraParams={}):
    params = {
        "name": name,
        **extraParams,
    }
    return entityRepository.editEntity(organization, params)


def newOrganization(user, name, extraParams={}):

    params = {
        "name": name,
        **extraParams,
    }

    return entityRepository.newEntity(user, Organization, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Thomas Bullier <thomas@labstep.com>

import labstep
import os
import labstep.generic.entity.repository as entityRepository
from labstep.entities.experimentProtocol.model import ExperimentProtocol


def getParent():
    """
    Get Parent based on Jupyter environment variables.

    """
    if ('LABSTEP_API_KEY' not in os.environ.keys() or 'LABSTEP_JUPYTER_EXPERIMENT_GUID' not in os.environ.keys()):
        raise Exception("Not in jupyter")

    user = labstep.authenticate()
    experimentGuid = os.environ['LABSTEP_JUPYTER_EXPERIMENT_GUID']

    return entityRepository.getEntity(user, ExperimentProtocol, experimentGuid, useGuid=True)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity
import labstep.generic.entity.repository as entityRepository
from labstep.service.helpers import getTime


class ProtocolDataField(Entity):
    """
    Represents a data field on a Labstep Protocol.

    To see the attributes of the data field run
    ::
        print(my_data_field)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(my_data_field.value)
        print(my_data_field.id)
    """

    __entityName__ = "metadata"
    __searchKey__ = "label"

    def edit(self, fieldName=None, value=None, extraParams={}):
        """
        Edit the value of an existing data field.

        Parameters
        ----------
        fieldName (str)
            The new name of the field.
        value (str)
            The new value of the data.

        Returns
        -------
        :class:`~labstep.entities.protocolDataField.model.ProtocolDataField`
            An object representing the edited data field.

        Example
        -------
        ::

            data.edit(value='2.50')
        """
        import labstep.entities.protocolDataField.repository as protocolDataFieldRepository

        return protocolDataFieldRepository.editDataField(
            self, fieldName, value, extraParams=extraParams
        )

    def delete(self):
        """
        Delete an existing Data field.

        Example
        -------
        ::

            data.delete()
        """
        import labstep.entities.protocolDataField.repository as protocolDataFieldRepository

        return protocolDataFieldRepository.editDataField(
            self, extraParams={"deleted_at": getTime()}
        )

    def linkToMaterial(self, material):
        """
        Link a data field to a material.

        Parameters
        ----------
        material :class:`~labstep.entities.protocolMaterial.model.ProtocolMaterial`
            The material to link the data field to.

        Example
        -------
        ::

            material = protocol.addMaterial('Sample')
            data = protocol.addDataField('Concentration')
            data.linkToMaterial(material)
        """
        return entityRepository.linkEntities(self.__user__, self, material)

    def getLinkedMaterials(self):
        """
        Returns the materials linked to this data field..

        Parameters
        ----------
        List[:class:`~labstep.entities.protocolMaterial.model.ProtocolMaterial`]
            The material link the data field to.

        Example
        -------
        ::

            material = protocol.addMaterial('Sample')
            data = protocol.addDataField('Concentration')
            data.linkToMaterial(material)
        """
        import labstep.entities.protocolMaterial.repository as protocolMaterialRepository

        if self.protocol_id is not None:

            return protocolMaterialRepository.getProtocolMaterials(
                user=self.__user__,
                protocol_id=self.protocol_id,
                extraParams={'metadata_id': self.id}
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Thomas Bullier <thomas@labstep.com>

from labstep.generic.primaryEntity.model import PrimaryEntity
from labstep.service.helpers import getTime


class JupyterInstance(PrimaryEntity):
    """
    Represents an JupyterInstance on Labstep.
    """

    __entityName__ = "jupyter-instance"
    __hasGuid__ = True

    def start(self):
        """
        Update JupyterInstance to indicate that server instance has started and is ready to use.

        Parameters
        ----------

        Returns
        -------
        :class:`~labstep.entities.jupyterInstance.model.JupyterInstance`
            An object representing the edited JupyterInstance.

        Example
        -------
        ::

            my_jupyterInstance = user.getJupyterInstance("872b3e7e-e21f-4403-9ef3-3650fe0d86ba")
            my_jupyterInstance.start()
        """
        import labstep.entities.jupyterInstance.repository as jupyterInstanceRepository

        return jupyterInstanceRepository.editJupyterInstance(
            self, startedAt=getTime()
        )

    def end(self):
        """
        Update JupyterInstance to indicate that server instance has shutdown.

        Parameters
        ----------

        Returns
        -------
        :class:`~labstep.entities.jupyterInstance.model.JupyterInstance`
            An object representing the edited JupyterInstance.

        Example
        -------
        ::

            my_jupyterInstance = user.getJupyterInstance("872b3e7e-e21f-4403-9ef3-3650fe0d86ba")
            my_jupyterInstance.end()
        """
        import labstep.entities.jupyterInstance.repository as jupyterInstanceRepository

        return jupyterInstanceRepository.editJupyterInstance(
            self, endedAt=getTime()
        )




from labstep.entities.experimentSignatureRequest.model import ExperimentSignatureRequest
import labstep.generic.entity.repository as entityRepository


def newExperimentSignatureRequest(user, experiment_id, user_id, message=None):
    fields = {
        "experiment_workflow_id": experiment_id,
        "message": message,
        "user_id": user_id
    }
    return entityRepository.newEntity(user, ExperimentSignatureRequest, fields=fields)


def getExperimentSignatureRequests(user, experiment_id):
    params = {
        "experiment_workflow_id": experiment_id,
        "search": None,
    }
    return entityRepository.getEntities(user, ExperimentSignatureRequest, filterParams=params, count=100)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import json
from labstep.service.config import configService
from labstep.service.helpers import url_join, getHeaders
from labstep.entities.file.model import File
from labstep.service.request import requestService
import labstep.generic.entity.repository as entityRepository


def newFile(user, filepath=None, rawData=None, extraParams={}):
    if filepath is not None:
        rawData = open(filepath, 'rb')
    if rawData is None:
        raise Exception('Please specify filepath or raw data')

    files = {'file': rawData}
    params = {"group_id": user.activeWorkspace, **extraParams}
    headers = getHeaders(user=user)
    url = url_join(configService.getHost(), "/api/generic/file/upload")
    response = requestService.post(
        url, headers=headers, files=files, data=params)
    data = json.loads(response.content)
    return File(list(data.values())[0], user)


def getFile(user, fileId):
    return entityRepository.getEntity(user, File, fileId, isDeleted=None)


def downloadFile(user, fileId):
    headers = getHeaders(user=user)
    url = url_join(configService.getHost(),
                   "/api/generic/file/download", str(fileId))
    response = requestService.post(url, headers=headers)
    rawData = requestService.get(json.loads(response.content)[
        "signed_url"], headers=None).content
    return rawData


def getFiles(
    user, count=100, search_query=None, file_type=None, extraParams={}
):
    params = {"search_query": search_query,
              "filetype": file_type, **extraParams}
    return entityRepository.getEntities(user, File, count, params)


def exportFile(file, root_path):

    fileDir = entityRepository.exportEntity(
        file, root_path, folderName=str(file.id))
    file.save(fileDir)
    if file.image_annotation is not None:
        annotatedFile = File(file.image_annotation, file.__user__)
        annotatedFile.export(root_path)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.service.helpers import (
    handleDate,
)
from labstep.entities.protocol.model import Protocol
from labstep.entities.protocolVersion.model import ProtocolVersion
import labstep.generic.entity.repository as entityRepository
from labstep.service.htmlExport import htmlExportService


def getProtocol(user, protocol_id):
    return entityRepository.getEntity(user, Protocol, id=protocol_id)


def getProtocols(

    user,
    count=100,
    search_query=None,
    created_at_from=None,
    created_at_to=None,
    tag_id=None,
    collection_id=None,
    extraParams={},
):
    params = {
        "search_query": search_query,
        "created_at_from": handleDate(created_at_from),
        "created_at_to": handleDate(created_at_to),
        "tag_id": tag_id,
        "folder_id": collection_id,
        **extraParams,
    }
    return entityRepository.getEntities(user, Protocol, count, params)


def newProtocol(user, name, extraParams={}):
    params = {"name": name, **extraParams}
    return entityRepository.newEntity(user, Protocol, params)


def editProtocol(
    protocol, name=None, body=None, deleted_at=None, extraParams={}
):
    params = {"name": name, "deleted_at": deleted_at, **extraParams}

    if body is not None:
        entityRepository.editEntity(
            ProtocolVersion(protocol.last_version, protocol.__user__),
            {"state": body},
        )
        protocol.update()

    return entityRepository.editEntity(protocol, params)


def exportProtocol(protocol, root_path):

    protocol.update()

    expDir = entityRepository.exportEntity(protocol, root_path)

    # export notes
    notesDir = expDir.joinpath('notes')
    notes = protocol.getComments(count=1000)

    for note in notes:
        note.export(notesDir)

    # save materials
    materialsDir = expDir.joinpath('materials')
    materials = protocol.getMaterials()

    for material in materials:
        material.export(materialsDir)

    # save data
    dataDir = expDir.joinpath('data')
    data = protocol.getDataFields()

    for dat in data:
        dat.export(dataDir)

    # get html
    html = htmlExportService.getHTML(protocol)
    html_with_paths = htmlExportService.insertFilepaths(expDir, html)

    with open(expDir.joinpath('entity.html'), 'w') as out:
        out.write(html_with_paths)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class ProtocolTimer(Entity):
    __entityName__ = "protocol-timer"

    def edit(self, name=None, hours=None, minutes=None, seconds=None, extraParams={}):
        """
        Edit an existing Protocol Timer.

        Parameters
        ----------
        name (str)
            The name of the timer.
        hours (int)
            The hours of the timer.
        minutes (int)
            The minutes of the timer.
        seconds (int)
            The seconds of the timer.

        Returns
        -------
        :class:`~labstep.entities.protocolTimer.model.ProtocolTimer`
            An object representing the edited Protocol Timer.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            protocol_timers = protocol.getTimers()
            protocol_timers[0].edit(name='New Timer Name',
                                    minutes=1, seconds=17)
        """
        import labstep.generic.entity.repository as entityRepository

        params = {"name": name, **extraParams}

        if hours is not None:
            params["hours"] = hours
        if minutes is not None:
            params["minutes"] = minutes
        if seconds is not None:
            params["seconds"] = seconds

        return entityRepository.editEntity(self, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.entities.workspaceMember.model import WorkspaceMember
import labstep.generic.entity.repository as entityRepository
from labstep.service.helpers import url_join, getHeaders
from labstep.service.config import configService
from labstep.service.request import requestService


def getMembers(user, workspace_id, count=100, search_query=None, extraParams={}):

    params = {"group_id": workspace_id,
              "search_query_user": search_query, **extraParams}

    return entityRepository.getEntities(user, WorkspaceMember, count, params)


def addMember(user, workspace_id, user_id):
    params = {
        "group_id": workspace_id,
        "user_id": user_id,
    }
    return entityRepository.newEntity(user, WorkspaceMember, params)


def editMemberPermission(member, permission):
    params = {
        "type": permission,
    }
    return entityRepository.editEntity(member, params)


def removeMember(member):
    url = url_join(configService.getHost(), 'api/generic',
                   'user-group', str(member.id))
    headers = getHeaders(member.__user__)
    return requestService.delete(url, headers)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.entities.resource.model import Resource
import labstep.generic.entity.repository as entityRepository


def getResource(user, resource_id):
    return entityRepository.getEntity(user, Resource, id=resource_id)


def getResources(
    user, count=100, search_query=None, tag_id=None, extraParams={}
):
    params = {"search_query": search_query,
              "tag_id": tag_id, **extraParams}
    return entityRepository.getEntities(user, Resource, count, params)


def newResource(user, name, resource_category_id=None, extraParams={}):
    params = {
        "name": name,
        "template_id": resource_category_id,
        **extraParams}
    return entityRepository.newEntity(user, Resource, params)


def editResource(

    resource,
    name=None,
    deleted_at=None,
    resource_category_id=None,
    extraParams={},
):
    params = {
        "name": name,
        "template_id": resource_category_id,
        "deleted_at": deleted_at,
        **extraParams,
    }
    return entityRepository.editEntity(resource, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import os
from labstep.generic.entity.model import Entity


class File(Entity):
    __entityName__ = "file"

    def getData(self):
        """
        Retrieve the contents of a File for manipulation within Python.

        Example
        -------
        ::

            entities = user.getFiles(search_query='bacteria')
            file = entities[0]
            data = file.getData()
        """
        import labstep.entities.file.repository as fileRepository

        return fileRepository.downloadFile(self.__user__, self.id)

    def save(self, folder=None, name=None):
        """
        Save a Labstep file to the local filesystem.

        Parameters
        ----------
        folder (str)
            The path to the folder where the file should be saved
            (defaults to the current working directory).
        name (str)
            Optionally give the file a new name.

        Returns
        -------
        None

        Example
        -------
        ::

            entities = user.getFiles(search_query='bacteria')
            file = entities[0]
            file.save()
        """
        if self.link_source is not None:
            print('Warning: Files from External Cloud Providers cannot be downloaded')
            return

        if name is None:
            name = self.name

        if folder is not None:
            filepath = os.path.join(folder, name)
        else:
            filepath = name

        data = self.getData()
        open(filepath, "wb").write(data)

    def export(self, path):
        """
        Export the file to the directory specified. 

        Parameters
        -------
        path (str)
            The path to the directory to save the file.

        Example
        -------
        ::

            file = user.getFile(17000)
            file.export('/my_folder')
        """
        import labstep.entities.file.repository as fileRepository

        return fileRepository.exportFile(self, path)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import labstep.generic.entity.repository as entityRepository


def exportExperimentProtocol(experimentProtocol, rootPath, folderName):

    experimentProtocol.update()

    expDir = entityRepository.exportEntity(
        experimentProtocol, rootPath, folderName=folderName)

    # save materials
    materialsDir = expDir.joinpath('materials')
    materials = experimentProtocol.getMaterials()

    for material in materials:
        material.export(materialsDir)

    # save data
    dataDir = expDir.joinpath('data')
    data = experimentProtocol.getDataFields()

    for dat in data:
        dat.export(dataDir)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import json
from labstep.entities.collection.model import Collection
from labstep.service.request import requestService
from labstep.service.helpers import url_join, getHeaders
from labstep.service.config import configService
from labstep.generic.entity.repository import getEntities, editEntity, newEntity


def getCollections(
    user, count=1000, type=None, search_query=None, extraParams={}
):
    types = {
        "experiment": "experiment_workflow",
        "protocol": "protocol_collection",
        None: None,
    }
    params = {"search_query": search_query, "type": types[type], **extraParams}
    return getEntities(user, Collection, count, params)


def getAttachedCollections(entity, count=100):
    key = entity.__entityName__.replace("-", "_") + "_id"
    filterParams = {key: entity.id,
                    "group_id": entity.__user__.activeWorkspace}
    return getEntities(
        entity.__user__, Collection, count=count, filterParams=filterParams
    )


def newCollection(user, name, type, extraParams={}):
    types = {"experiment": "experiment_workflow",
             "protocol": "protocol_collection"}
    params = {"name": name, "type": types[type], **extraParams}
    return newEntity(user, Collection, params)


def addToCollection(entity, collection_id):
    entityName = entity.__entityName__

    headers = getHeaders(entity.__user__)
    url = url_join(
        configService.getHost(),
        "api/generic/",
        entityName,
        str(entity.id),
        Collection.__entityName__,
        str(collection_id),
    )
    response = requestService.put(url, headers=headers)
    return json.loads(response.content)


def removeFromCollection(entity, collection_id):
    entityName = entity.__entityName__

    headers = getHeaders(entity.__user__)
    url = url_join(
        configService.getHost(),
        "api/generic/",
        entityName,
        str(entity.id),
        Collection.__entityName__,
        str(collection_id),
    )
    response = requestService.delete(url, headers=headers)
    return json.loads(response.content)


def editCollection(collection, name, extraParams={}):
    params = {"name": name, **extraParams}
    return editEntity(collection, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class ProtocolMaterial(Entity):
    __entityName__ = "protocol-value"

    def __init__(self, data, user):
        super().__init__(data, user)
        self.amount = self.value

    def edit(
        self, name=None, amount=None, units=None, resource_id=None, extraParams={}
    ):
        """
        Edit an existing Protocol Material.

        Parameters
        ----------
        name (str)
            The name of the Protocol Material.
        amount (str)
            The amount of the Protocol Material.
        units (str)
            The units of the amount.
        resource_id (Resource)
            The id of the :class:`~labstep.entities.resource.model.Resource` of
            the Protocol Material.

        Returns
        -------
        :class:`~labstep.entities.protocolMaterial.model.ProtocolMaterial`
            An object representing the edited Protocol Material.

        Example
        -------
        ::

            protocol = user.getProtocol(17000)
            protocol_materials = exp_protocol.getMaterials()
            protocol_materials[0].edit(value=1.7, units='ml')
        """
        import labstep.entities.protocolMaterial.repository as protocolMaterialRepository

        return protocolMaterialRepository.editProtocolMaterial(self,
                                                               name=name,
                                                               amount=amount,
                                                               units=units,
                                                               resource_id=resource_id,
                                                               extraParams=extraParams)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import json
import glob
from bs4 import BeautifulSoup
from labstep.service.request import requestService
from labstep.service.config import configService
from labstep.service.helpers import (
    url_join,
    getHeaders,
)


class HTMLExportService:
    def getHTML(self, entity):
        """
        Gets HTML summary of experiments / protocols.

        """
        headers = getHeaders(entity.__user__)

        body = {
            "group_id": entity.__user__.activeWorkspace,
            "query_entity_name": entity.__entityName__.replace("-", "_"),
            "query_parameters": {
                "id": entity.id
            },
            "type": "html"
        }

        url = url_join(configService.getHost(), 'api/generic', 'entity-export')
        response = requestService.post(url, json=body, headers=headers)

        return json.loads(response.content)['html']

    def insertFilepaths(self, rootDir, html):

        soup = BeautifulSoup(html, 'html.parser')

        for img in soup.find_all('img', {'data-file-id': True, 'data-file-is-link': False}):
            file_id = img['data-file-id']
            file_name = glob.escape(img['data-file-name'])
            path = list(rootDir.glob(f'**/files/{file_id}*/{file_name}'))
            img['src'] = str(path[0].relative_to(rootDir))

        for a in soup.find_all('a', {'data-file-id': True, 'data-file-is-link': False}):
            file_id = a['data-file-id']
            file_name = glob.escape(a['data-file-name'])
            path = list(rootDir.glob(f'**/files/{file_id}*/{file_name}'))
            a['href'] = str(path[0].relative_to(rootDir))

        return str(soup)


htmlExportService = HTMLExportService()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# pylama:ignore=W0611
# Author: Barney Walker <barney@labstep.com>

"""Labstep Python SDK"""

from .entities.user.facade import login, authenticate, impersonate
from .service import jupyter
from .service.ping import ping



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.entities.resourceLocation.model import ResourceLocation
import labstep.generic.entity.repository as entityRepository


def getResourceLocation(user, resource_location_id):
    return entityRepository.getEntity(
        user, ResourceLocation, id=resource_location_id
    )


def getResourceLocations(
    user, count=100, search_query=None, tag_id=None, extraParams={}
):
    params = {
        "group_id": user.activeWorkspace,
        "search_query": search_query,
        "tag_id": tag_id,
        **extraParams,
    }
    return entityRepository.getEntities(user, ResourceLocation, count, params)


def newResourceLocation(user, name, outer_location_id=None, extraParams={}):
    params = {"name": name, "outer_location_id": outer_location_id, **extraParams}
    return entityRepository.newEntity(user, ResourceLocation, params)


def editResourceLocation(resourceLocation, name, extraParams={}):
    params = {"name": name, **extraParams}
    return entityRepository.editEntity(resourceLocation, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity
import labstep.generic.entity.repository as entityRepository
from labstep.service.helpers import getTime


class ExperimentDataField(Entity):
    """
    Represents a single data field attached to a Labstep Experiment.

    To see the attributes of the data field run
    ::
        print(my_data_field)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(my_data_field.value)
        print(my_data_field.id)
    """

    __entityName__ = "metadata"
    __searchKey__ = 'label'

    def edit(self, fieldName=None, value=None, extraParams={}):
        """
        Edit the value of an existing data field.

        Parameters
        ----------
        fieldName (str)
            The new name of the field.
        value (str)
            The new value of the data.

        Returns
        -------
        :class:`~labstep.entities.experimentDataField.model.ExperimentDataField`
            An object representing the edited data field.

        Example
        -------
        ::

            data.edit(value='2.50')
        """
        import labstep.entities.experimentDataField.repository as experimentDataFieldRepository

        return experimentDataFieldRepository.editDataField(
            self, fieldName, value, extraParams=extraParams
        )

    def delete(self):
        """
        Delete an existing Data field.

        Example
        -------
        ::

            data.delete()
        """
        import labstep.entities.experimentDataField.repository as experimentDataFieldRepository

        return experimentDataFieldRepository.editDataField(
            self, extraParams={"deleted_at": getTime()}
        )

    def linkToMaterial(self, material):
        """
        Link a data field to a material.

        Parameters
        ----------
        material
            The :class:`~labstep.entities.experimentMaterial.model.ExperimentMaterial` to link the data field to.

        Example
        -------
        ::

            material = experiment.addMaterial('Sample')
            data = experiment.addDataField('Concentration')
            data.linkToMaterial(material)
        """
        return entityRepository.linkEntities(self.__user__, self, material)

    def getLinkedMaterials(self):
        """
        Returns the materials linked to this data field..

        Returns
        ----------
        List[:class:`~labstep.entities.experimentMaterial.model.ExperimentMaterial`]
            The material link the data field to.

        Example
        -------
        ::

            material = experiment.addMaterial('Sample')
            data = experiment.addDataField('Concentration')
            data.linkToMaterial(material)
        """
        import labstep.entities.experimentMaterial.repository as experimentMaterialRepository

        if self.experiment_id is not None:

            return experimentMaterialRepository.getExperimentMaterials(
                user=self.__user__,
                experiment_id=self.experiment_id,
                extraParams={'metadata_id': self.id}
            )

    def getValue(self):
        """
        Returns the value of the data field.

        Returns
        ----------
        Return type depends on the data type of the data field

        Example
        -------
        ::

            dataField = experiment.getDataFields()[0]
            value = dataField.getValue()
        """

        import labstep.entities.experimentDataField.repository as experimentDataFieldRepository

        return experimentDataFieldRepository.getDataFieldValue(self)

    def setValue(self, value):
        """
        Sets the value of the data field.

        Parameters
        ----------
        value 
            The value to set, depends on the type of the data field. 

        Returns
        ----------
        Return type depends on the data type of the data field

        Example
        -------
        ::

            dataFields = experiment.getDataFields()
            textField = dataFields.get('My text field')
            textField.setValue('Some String')

            numericField = dataFields.get('My numeric field')
            numericField.setValue(56534)

            dateField = dataFields.get('My date field')
            dateField.setValue('2021-10-28')

            singleOptionsField = dataFields.get('My single options field')
            singleOptionsField.setValue('Option 1')

            multiOptionsField = dataFields.get('My multi options field')
            multiOptionsField.setValue(['Option 1','Option 2'])

            fileField = dataFields.get('My file field')
            file = user.newFile('/path/to/file')
            fileField.setValue(file)

        """

        import labstep.entities.experimentDataField.repository as experimentDataFieldRepository

        return experimentDataFieldRepository.setDataFieldValue(self, value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.service.helpers import url_join, getHeaders
from labstep.service.config import configService
from labstep.service.request import requestService
from labstep.entities.invitation.model import Invitation
import labstep.generic.entity.repository as entityRepository


def newInvitations(user, invitationType, emails, organization_id, workspace_id=None):
    url = url_join(configService.getHost(), 'api/generic',
                   'share-link-invitation', invitationType)
    headers = getHeaders(user=user)
    json = {
        'emails': emails,
        'organization_id': organization_id,
        'organization_group_id': workspace_id
    }
    requestService.post(url=url, headers=headers, json=json)


def getInvitations(user, organization_id, extraParams):
    return entityRepository.getEntities(user, Invitation, count=None, filterParams={
        'organization_id': organization_id, **extraParams})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class ProtocolVersion(Entity):
    __entityName__ = "protocol"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.entities.comment.model import Comment
from labstep.generic.entity.repository import getEntities, newEntity, editEntity, exportEntity
import labstep.entities.file.repository as fileRepository
from labstep.service.helpers import handleString


def getComments(entity, count=100, extraParams={}):

    if entity.thread is None:
        return []

    params = {
        "parent_thread_id": entity.thread["id"],
        "search": None,
        **extraParams,
    }
    return getEntities(entity.__user__, Comment, count, params)


def addComment(entity, body, fileId=None, extraParams={}):
    if entity.thread is None:
        raise Exception('You cannot add a comment to this entity')

    threadId = entity.thread["id"]

    params = {
        "body": handleString(body),
        "parent_thread_id": threadId,
        "file_id": [[fileId]] if fileId is not None else None,
        **extraParams,
    }

    return newEntity(entity.__user__, Comment, params)


def addCommentWithFile(entity, body, filepath, extraParams={}):
    if filepath is not None:
        fileId = fileRepository.newFile(entity.__user__, filepath).id
    else:
        fileId = None
    return addComment(entity, body, fileId=fileId, extraParams=extraParams)


def editComment(comment, body=None, extraParams={}):
    params = {"body": body, **extraParams}
    return editEntity(comment, params)


def exportComment(comment, rootPath):
    commentDir = exportEntity(
        comment, rootPath)

    # export comment
    nestedCommentsDir = commentDir.joinpath('comments')
    nestedComments = comment.getComments(count=1000)

    for nestedComment in nestedComments:
        nestedComment.export(
            nestedCommentsDir)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.entities.sharelink.model import Sharelink
import labstep.generic.entity.repository as entityRepository


def getSharelink(entity):
    key = entity.__entityName__.replace("-", "_") + "_id"
    return entityRepository.newEntity(entity.__user__, Sharelink, fields={key: entity.id})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.service.helpers import (
    handleDate,
)
from labstep.entities.experiment.model import Experiment, ExperimentProtocol
import labstep.generic.entity.repository as entityRepository
from labstep.service.htmlExport import htmlExportService


def getExperiment(user, experiment_id):
    return entityRepository.getEntity(user, Experiment, id=experiment_id)


def getExperiments(

    user,
    count=100,
    search_query=None,
    created_at_from=None,
    created_at_to=None,
    tag_id=None,
    collection_id=None,
    extraParams={},
):
    params = {
        "search_query": search_query,
        "created_at_from": handleDate(created_at_from),
        "created_at_to": handleDate(created_at_to),
        "tag_id": tag_id,
        "folder_id": collection_id,
        **extraParams,
    }
    return entityRepository.getEntities(user, Experiment, count, params)


def newExperiment(user, name, entry=None, extraParams={}):
    params = {"name": name, **extraParams}

    experiment = entityRepository.newEntity(user, Experiment, params)

    if entry is not None:
        experiment = experiment.edit(entry=entry)

    return experiment


def addProtocolToExperiment(experiment, protocol):
    params = {
        "experiment_workflow_id": experiment.id,
        "protocol_id": protocol.last_version["id"],
    }
    return entityRepository.newEntity(
        experiment.__user__, ExperimentProtocol, params
    )


def editExperiment(

    experiment,
    name=None,
    entry=None,
    started_at=None,
    deleted_at=None,
    extraParams={},
):
    params = {
        "name": name,
        "started_at": handleDate(started_at),
        "deleted_at": deleted_at,
        **extraParams,
    }

    if entry is not None:
        experiment.root_experiment.edit(body=entry)
        experiment.update()

    return entityRepository.editEntity(experiment, params)


def exportExperiment(experiment, root_path):

    experiment.update()

    expDir = entityRepository.exportEntity(
        experiment, root_path)

    # export entry
    experiment.root_experiment.export(expDir, folderName='entry')

    # export protocols
    protocolsDir = expDir.joinpath('protocols')
    protocols = experiment.getProtocols(count=100)

    for protocol in protocols:
        protocol.export(protocolsDir)

    # export notes
    notesDir = expDir.joinpath('notes')
    notes = experiment.getComments(count=1000)

    for note in notes:
        note.export(notesDir)

    # get html
    html = htmlExportService.getHTML(experiment)
    html_with_paths = htmlExportService.insertFilepaths(expDir, html)

    with open(expDir.joinpath('entity.html'), 'w', encoding="utf-8") as out:
        out.write(html_with_paths)



class EntityList(list):
    def __init__(self, items, entityClass, user):
        super().__init__(map(lambda x: entityClass(x, user), items))
        self.__searchKey__ = getattr(entityClass, '__searchKey__', 'name')

    def get(self, key):
        hits = [entity for entity in self if getattr(
            entity, self.__searchKey__) == key]

        if len(hits) == 0:
            return None

        if len(hits) == 1:
            return hits[0]

        if len(hits) > 1:
            print('Warning: multiple matches found')

            return hits[0]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from datetime import datetime
from time import gmtime, strftime
from labstep.constants.version import VERSION


def url_join(*args):
    """
    Join a set of args with a slash (/) between them. Instead of
    checking for the presence of a slash, the slashes from both sides
    of each arg will be .strip() and then args .join() together.

    Returns
    -------
        returns "/".join(arg.strip("/") for arg in args)

    """
    return "/".join(arg.strip("/") for arg in args)


def handleError(response, *args, **kwargs):
    """
    Returns
    -------
        The error code and error message if the
        status code of the request is not 200.
    """
    if response.status_code != 200:
        print(
            """Get the latest version of the SDK by running:
        pip install labstep --upgrade"""
        )
        raise Exception(
            "Request Error {code}:{message}".format(
                code=response.status_code, message=response.content
            )
        )
    return


def getTime():
    """
    Returns
    -------
    timestamp
        The datetime at .now() in json serializable format.
    """
    timezone = strftime("%z", gmtime())
    tz_hour = timezone[:3]
    tz_minute = timezone[3:]
    timestamp = datetime.now().strftime(
        "%Y-%m-%d" + "T" + "%H:%M:%S" + "{}:{}".format(tz_hour, tz_minute)
    )
    return timestamp


def handleDate(date):
    """
    Returns
    -------
        The datetime for in json serializable format.
    """
    if date is None:
        return None

    for fmt in ('%Y-%m-%d', '%Y-%m-%d %H:%M', '%Y-%m-%d %H:%M:%S'):
        try:
            return datetime.strptime(date, fmt).strftime('%Y-%m-%dT%H:%M:%S+00:00')
        except ValueError:
            pass
    raise ValueError(
        'Please Specify date in format: YY-MM-DD or YY-MM-DD HH:MM:SS')



def handleString(string):
    """
    Casts to string or returns None
    """
    return str(string) if string else None


def handleKeyword(string):
    """
    Returns
    -------
    string
        The string in lowercase, and any whitespace replaced with '_'.
    """
    if string is None:
        return None
    else:
        return string.lower().replace(" ", "_")


def update(entity, newData):
    """
    Returns
    -------
    the updated entity
    """
    entity.__data__ = newData
    for key in newData:
        setattr(entity, key, newData[key])
    return entity


def getHeaders(user=None):
    if user is None:
        return {
            "User-Agent": f"Python SDK {VERSION}"
        }
    else:
        return {
            "apikey": user.api_key,
            "User-Agent": f"Python SDK {VERSION}"
        }



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import json
from labstep.service.request import requestService


class HTMLToProseMirrorService:
    def parseHTML(self, authenticatedUser, html):
        """
        Converts HTML into the JSON format used in protocols
        and experiment entries.

        Parameters
        ----------
        authenticatedUser (:)
            An authenticated Labstep :class:`~labstep.entities.user.model.User`
            (see :func:`~labstep.authenticate`)
        html (str)
            HTML to be converted (as a string).
        """
        headers = {"Authorization": f"Bearer {authenticatedUser.token}"}

        body = {"html": html}

        url = "https://html-converter.labstep.com"
        response = requestService.post(url, json=body, headers=headers)
        return json.loads(response.content)


htmlToProseMirrorService = HTMLToProseMirrorService()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class Tag(Entity):
    """
    Represents a Tag on Labstep.

    To see all attributes of a tag run
    ::
        print(my_tag)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(my_tag.name)
        print(my_tag.id)
    """

    __entityName__ = "tag"
    __hasParentGroup__ = True

    def edit(self, name=None, extraParams={}):
        """
        Edit the name of an existing Tag.

        Parameters
        ----------
        name (str)
            The new name of the Tag.

        Returns
        -------
        :class:`~labstep.entities.tag.model.Tag`
            An object representing the edited Tag.

        Example
        -------
        ::

            # Get all tags, since there is no function
            # to get one tag.
            tags = user.getTags()

            # Select the tag by using python index.
            tags[1].edit(name='A New Tag Name')
        """
        import labstep.entities.tag.repository as tagRepository

        return tagRepository.editTag(self, name, extraParams=extraParams)

    def delete(self):
        """
        Delete an existing tag.

        Parameters
        ----------
        tag (obj)
            The tag to delete.

        Returns
        -------
        tag
            An object representing the tag to delete.
        """
        import labstep.entities.tag.repository as tagRepository

        return tagRepository.deleteTag(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity
from labstep.service.helpers import getTime


class ResourceItem(Entity):
    """
    Represents a Resource Item on Labstep.

    To see all attributes of the resource item run
    ::
        print(my_resource_item)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(my_resource_item.name)
        print(my_resource_item.id)
    """

    __entityName__ = "resource-item"

    def edit(
        self,
        name=None,
        availability=None,
        quantity_amount=None,
        quantity_unit=None,
        resource_location_id=None,
        extraParams={},
    ):
        """
        Edit an existing ResourceItem.

        Parameters
        ----------
        name (str)
            The new name of the ResourceItem.
        availability (str)
            The availability of the ResourceItem. Options are:
            "available" and "unavailable".
        quantity_amount (float)
            The quantity of the ResourceItem.
        quantity_unit (str)
            The unit of the quantity.
        resource_location_id (int)
            The id of the :class:`~labstep.entities.resourceLocation.model.ResourceLocation` of the ResourceItem.

        Returns
        -------
        :class:`~labstep.entities.resourceItem.model.ResourceItem`
            An object representing the edited ResourceItem.

        Example
        -------
        ::

            my_resource_item = user.getResourceItem(17000)
            my_resource_item.edit(name='A New ResourceItem Name')
        """
        import labstep.entities.resourceItem.repository as resourceItemRepository

        return resourceItemRepository.editResourceItem(
            self,
            name=name,
            availability=availability,
            quantity_amount=quantity_amount,
            quantity_unit=quantity_unit,
            resource_location_id=resource_location_id,
            extraParams=extraParams,
        )

    def delete(self):
        """
        Delete an existing ResourceItem.

        Example
        -------
        ::

            my_resource_item = user.getResourceItem(17000)
            my_resource_item.delete()
        """
        import labstep.entities.resourceItem.repository as resourceItemRepository

        return resourceItemRepository.editResourceItem(self, deleted_at=getTime())

    def addComment(self, body, filepath=None, extraParams={}):
        """
        Add a comment and/or file to a Labstep ResourceItem.

        Parameters
        ----------
        body (str)
            The body of the comment.
        filepath (str)
            A Labstep File entity to attach to the comment,
            including the filepath.

        Returns
        -------
        :class:`~labstep.entities.comment.model.Comment`
            The comment added.

        Example
        -------
        ::

            my_resource_item = user.getResourceItem(17000)
            my_resource_item.addComment(body='I am commenting!',
                                        filepath='pwd/file_to_upload.dat')
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.addCommentWithFile(
            self, body, filepath, extraParams=extraParams
        )

    def getComments(self, count=100, extraParams={}):
        """
        Retrieve the Comments attached to this Labstep Entity.

        Returns
        -------
        List[:class:`~labstep.entities.comment.model.Comment`]
            List of the comments attached.

        Example
        -------
        ::

            entity = user.getResource(17000)
            item = entity.getItems()[1]
            comments = item.getComments()
            comments[0].attributes()
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.getComments(self, count, extraParams=extraParams)

    def addMetadata(
        self,
        fieldName,
        fieldType="default",
        value=None,
        date=None,
        number=None,
        unit=None,
        filepath=None,
        extraParams={},
    ):
        """
        Add Metadata to a ResourceItem.

        Parameters
        ----------
        fieldName (str)
            The name of the field.
        fieldType (str)
            The Metadata field type. Options are: "default", "date",
            "numeric", or "file". The "default" type is "Text".
        value (str)
            The value accompanying the fieldName entry.
        date (str)
            The date accompanying the fieldName entry. Must be
            in the format of "YYYY-MM-DD HH:MM".
        number (float)
            The numeric value.
        unit (str)
            The unit accompanying the number entry.
        filepath (str)
            Local path to the file to upload for type 'file'

        Returns
        -------
        :class:`~labstep.entities.metadata.model.Metadata`
            An object representing the new Labstep Metadata.

        Example
        -------
        ::

            my_resource_item = user.getResourceItem(17000)
            metadata = my_resource_item.addMetadata("Refractive Index",
                                                    value="1.73")
        """
        import labstep.entities.metadata.repository as metadataRepository

        return metadataRepository.addMetadataTo(
            self,
            fieldName,
            fieldType=fieldType,
            value=value,
            date=date,
            number=number,
            unit=unit,
            filepath=filepath,
            extraParams=extraParams,
        )

    def getMetadata(self):
        """
        Retrieve the Metadata of a Labstep ResourceItem.

        Returns
        -------
        List[:class:`~labstep.entities.resource.model.ResourceItem`]
            An array of Metadata objects for the ResourceItem.

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            my_resource_item = my_resource.getResourceItem(17000)
            metadata = my_resource_item.getMetadata()
            metadatas[0].attributes()
        """
        import labstep.entities.metadata.repository as metadataRepository

        return metadataRepository.getMetadata(self)

    def getData(self):
        """
        Returns data linked to this resource via experiments.

        Returns
        -------
        List[:class:`~labstep.entities.experimentDataField.model.ExperimentDataField`]


        Example
        -------
        ::

            my_resource = user.getResource(17000)
            my_resource.getData()
        """
        import labstep.entities.experimentDataField.repository as experimentDataFieldRepository

        return experimentDataFieldRepository.getDataFields(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Thomas Bullier <thomas@labstep.com>
# TODO Implement routing name
# Example: url = url_join(configService.getHost(), "api/generic/share-link/email")

import os
import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
from labstep.service.helpers import handleError

DEFAULT_TIMEOUT = 60  # seconds


class TimeoutHTTPAdapter(HTTPAdapter):
    def __init__(self, *args, **kwargs):
        self.timeout = DEFAULT_TIMEOUT
        if "timeout" in kwargs:
            self.timeout = kwargs["timeout"]
            del kwargs["timeout"]
        super().__init__(*args, **kwargs)

    def send(self, request, **kwargs):
        timeout = kwargs.get("timeout")
        if ("DISABLE_SSL_VERIFY" in os.environ.keys() and os.environ["DISABLE_SSL_VERIFY"] == "1"):
            kwargs['verify'] = False
        if timeout is None:
            kwargs["timeout"] = self.timeout
        return super().send(request, **kwargs)


http = requests.Session()

retry_strategy = Retry(
    total=3,
    status_forcelist=[501, 502, 503, 504],
    raise_on_status=False
)
# Mount it for both http and https usage
adapter = TimeoutHTTPAdapter(timeout=60, max_retries=retry_strategy)
http.hooks["response"] = [handleError]
http.mount("https://", adapter)
http.mount("http://", adapter)


def encode_boolean_values(kv):
    """ convert bool values to 'true'/'false' strings for json compat """
    def enc(x): return x if not isinstance(
        x, bool) else 'true' if x else 'false'

    return None if kv is None else {k: enc(v) for k, v in kv.items()}


class RequestService:
    def get(self, url, headers, params=None):
        response = http.get(url, headers=headers,
                            params=encode_boolean_values(params))
        return response

    def post(self, url, headers, json=None, files=None, data=None):
        response = http.post(
            url, headers=headers, json=json, files=files, data=data
        )
        return response

    def put(self, url, headers, json=None):
        response = http.put(url, headers=headers, json=json)
        return response

    def delete(self, url, headers, json=None):
        response = http.delete(url, headers=headers, json=json)
        return response


requestService = RequestService()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity
from labstep.service.helpers import getTime


class ExperimentSignature(Entity):
    __entityName__ = "signature"

    def revoke(self):
        """
        Revokes the signature.

        Returns
        -------
        :class:`~labstep.entities.experimentSignature.model.ExperimentSignature`
            An object representing the revoked signature.
        """
        import labstep.generic.entity.repository as entityRepository

        fields = {"revoked_at": getTime()}
        return entityRepository.editEntity(self, fields)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import json
from labstep.service.config import configService
from labstep.service.helpers import url_join, handleKeyword, getHeaders
from labstep.entities.tag.model import Tag
from labstep.service.request import requestService
import labstep.generic.entity.repository as entityRepository


def getTags(user, count=1000, type=None, search_query=None, extraParams={}):
    """
    Retrieve a list of the user's tags.

    Parameters
    ----------
    user (obj)
        The Labstep user.
        Must have property 'api_key'. See 'login'.
    count (int)
        The number of Tags to retrieve.
    type (str)
        Return only tags of a certain type. Options are:
        'experiment_workflow', 'protocol_collection',
        'resource', 'order_request'.
    search_query (str)
        Search for Tags with this 'name'.
    extraParams (dict)
        Dictionary of extra filter parameters.

    Returns
    -------
    tags
        A list of tag objects.
    """
    params = {
        "search_query": search_query,
        "type": handleKeyword(type),
        **extraParams,
    }
    return entityRepository.getEntities(user, Tag, count, params)


def getAttachedTags(entity, count=100):
    """
    Retrieve the Tags attached to a Labstep Entity.

    Parameters
    ----------
    entity (obj)
        The entity to retrieve Tags from.

    Returns
    -------
    tags
        List of the tags attached.
    """
    key = entity.__entityName__.replace("-", "_") + "_id"
    filterParams = {key: entity.id}
    return entityRepository.getEntities(
        entity.__user__, Tag, count=count, filterParams=filterParams
    )


def newTag(user, name, type, extraParams={}):
    """
    Create a new Tag.

    Parameters
    ----------
    user (obj)
        The Labstep user creating the Tag.
        Must have property 'api_key'. See 'login'.
    name (str)
        Name of the new Tag.
    type (str)
            Return only tags of a certain type. Options are:
            'experiment_workflow', 'protocol_collection',
            'resource', 'order_request'.

    Returns
    -------
    tag
        An object representing the new Labstep Tag.
    """
    params = {"name": name, "type": handleKeyword(type), **extraParams}
    return entityRepository.newEntity(user, Tag, params)


def addTagTo(entity, tag):
    """
    Attach an existing tag to a Labstep entity.
    (See 'tag' for simplified tagging).

    Parameters
    ----------
    entity (obj)
        The Labstep entity to tag. Can be Resource,
        Experiment, or Protocol. Must have 'id'.
    tag (obj)
        The tag to attach. Must have an 'id' property.

    Returns
    -------
    entity
        An object representing the tagged entity.
    """
    entityName = entity.__entityName__

    headers = getHeaders(entity.__user__)
    url = url_join(
        configService.getHost(),
        "api/generic/",
        entityName,
        str(entity.id),
        tag.__entityName__,
        str(tag.id),
    )
    response = requestService.put(url, headers=headers)
    return json.loads(response.content)


def tag(entity, name):
    """
    Simple tagging of a Labstep entity (creates a
    new tag if none exists).

    Parameters
    ----------
    entity (obj)
        The Labstep entity to tag. Can be Resource,
        Experiment, or Protocol. Must have 'id'.
    name (str)
        The name of the tag to create.

    Returns
    -------
    entity
        An object representing the tagged entity.
    """
    user = entity.__user__
    type = (entity.__entityName__).replace("-", "_")

    tags = getTags(
        user,
        type=type,
        search_query=name,
        extraParams={"group_id": user.activeWorkspace},
    )
    matchingTags = list(filter(lambda x: x.name.lower() == name.lower(), tags))

    if len(matchingTags) == 0:
        tag = newTag(user, name, type=type)
    else:
        tag = matchingTags[0]

    return addTagTo(entity, tag)


def editTag(tag, name, extraParams={}):
    """
    Edit the name of an existing Tag.

    Parameters
    ----------
    tag (obj)
        The Tag to edit.
    name (str)
        The new name of the Tag.

    Returns
    -------
    tag
        An object representing the edited Tag.
    """
    params = {"name": name, **extraParams}
    return entityRepository.editEntity(tag, params)


def deleteTag(tag):
    headers = getHeaders(tag.__user__)
    url = url_join(configService.getHost(), "/api/generic/",
                   Tag.__entityName__, str(tag.id))
    requestService.delete(url, headers=headers)
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Thomas Bullier <thomas@labstep.com>

from labstep.entities.jupyterInstance.model import JupyterInstance
import labstep.generic.entity.repository as entityRepository


def getJupyterInstance(user, guid):
    return entityRepository.getEntity(user, JupyterInstance, id=guid)


def editJupyterInstance(
    jupyterInstance,
    startedAt=None,
    endedAt=None
):
    params = {
        "started_at": startedAt,
        "ended_at": endedAt,
    }

    return entityRepository.editEntity(jupyterInstance, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Thomas Bullier <thomas@labstep.com>

from labstep.entities.jupyterNotebook.model import JupyterNotebook
import labstep.generic.entity.repository as entityRepository


def newJupyterNotebook(user, name=None, data=None):
    return entityRepository.newEntity(user, JupyterNotebook, {"name": name, "data": data})


def getJupyterNotebook(user, guid):
    return entityRepository.getEntity(user, JupyterNotebook, id=guid)


def editJupyterNotebook(
    jupyterNotebook,
    name=None,
    status=None,
    data=None,
    extraParams={},
):
    params = {
        "name": name,
        "status": status,
        "data": data,
        **extraParams,
    }

    return entityRepository.editEntity(jupyterNotebook, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.service.helpers import getTime
from labstep.entities.organizationUser.model import OrganizationUser
import labstep.generic.entity.repository as entityRepository
from labstep.entities.user.model import User


def getOrganizationUsers(organization, count=100, extraParams={}):

    params = {
        'organization_id': organization.id,
        **extraParams,
    }

    return entityRepository.getEntities(
        organization.__user__,
        OrganizationUser, count,
        filterParams=params)


def promoteUser(organizationUser):
    params = {
        "type": "admin",
    }
    return entityRepository.editEntity(organizationUser, params)


def demoteUser(organizationUser):
    params = {
        "type": "member",
    }
    return entityRepository.editEntity(organizationUser, params)


def disableUser(organizationUser):
    params = {
        "deleted_at": getTime(),
    }
    user = User({"id": organizationUser.user['username']})
    user.__user__ = organizationUser.__user__
    return entityRepository.editEntity(user, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import json
from labstep.service.helpers import url_join, getHeaders
from labstep.service.config import configService
from labstep.service.request import requestService
from labstep.entities.permission.model import Permission
from labstep.generic.entityList.model import EntityList


def newPermission(entity, workspace_id, permission):
    entityName = entity.__entityName__

    headers = getHeaders(entity.__user__)
    url = url_join(configService.getHost(), "api/generic/", "acl")

    params = {
        "id": entity.id,
        "entity_class": entityName.replace("-", "_"),
        "action": "grant",
        "group_id": workspace_id,
        "permission": permission,
    }
    requestService.post(url, headers=headers, json=params)
    return entity


def editPermission(entity, workspace_id, permission):
    entityName = entity.__entityName__

    headers = getHeaders(entity.__user__)
    url = url_join(configService.getHost(), "api/generic/", "acl")

    params = {
        "id": entity.id,
        "entity_class": entityName.replace("-", "_"),
        "action": "set",
        "group_id": workspace_id,
        "group_owner_id": workspace_id,
        "permission": permission,
    }
    requestService.post(url, headers=headers, json=params)
    return entity


def revokePermission(entity, workspace_id):
    entityName = entity.__entityName__

    headers = getHeaders(entity.__user__)
    url = url_join(configService.getHost(), "api/generic/", "acl")

    params = {
        "id": entity.id,
        "entity_class": entityName.replace("-", "_"),
        "action": "revoke",
        "group_id": workspace_id,
    }
    requestService.post(url, headers=headers, json=params)
    return entity


def getPermissions(entity):
    entityName = entity.__entityName__
    headers = getHeaders(entity.__user__)
    url = url_join(
        configService.getHost(),
        "api/generic/",
        "acl",
        entityName.replace("-", "_"),
        str(entity.id),
    )
    response = requestService.get(url, headers=headers)
    resp = json.loads(response.content)
    return EntityList(resp["group_permissions"], Permission, entity)


def transferOwnership(entity, workspace_id):
    entityName = entity.__entityName__
    headers = getHeaders(entity.__user__)
    url = url_join(
        configService.getHost(), "api/generic/", entityName, str(entity.id), "transfer-ownership"
    )
    params = {"group_id": workspace_id}
    requestService.post(url, headers=headers, json=params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class OrganizationUser(Entity):
    __entityName__ = "user-organization"

    def setAdmin(self):
        """
        Gives a user admin permission over the organization.


        Example
        -------
        ::

            users = my_organization.getUsers()
            user[0].setAdmin()
        """
        import labstep.entities.organizationUser.repository as organizationUserRepository

        return organizationUserRepository.promoteUser(self)

    def revokeAdmin(self):
        """
        Revokes admin permissions from a user.


        Example
        -------
        ::

            users = my_organization.getUsers()
            user[0].revokeAdmin()
        """
        import labstep.entities.organizationUser.repository as organizationUserRepository

        return organizationUserRepository.demoteUser(self)

    def disable(self):
        """
        Disables a user account.


        Example
        -------
        ::

            users = my_organization.getUsers()
            user[0].disable()
        """
        import labstep.entities.organizationUser.repository as organizationUserRepository

        return organizationUserRepository.disableUser(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.service.helpers import getTime
from labstep.generic.primaryEntity.model import PrimaryEntity
from labstep.entities.resourceItem.model import ResourceItem


class Resource(PrimaryEntity):
    """
    Represents a Resource on Labstep.

    To see all attributes of the resource run
    ::
        print(my_resource)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(my_resource.name)
        print(my_resource.id)
    """

    __entityName__ = "resource"

    def edit(self, name=None, extraParams={}):
        """
        Edit an existing Resource.

        Parameters
        ----------
        name (str)
            The new name of the Resource.

        Returns
        -------
        :class:`~labstep.entities.resource.model.Resource`
            An object representing the edited Resource.

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            my_resource.edit(name='A New Resource Name')
        """
        import labstep.entities.resource.repository as resourceRepository

        return resourceRepository.editResource(self, name, extraParams=extraParams)

    def delete(self):
        """
        Delete an existing Resource.

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            my_resource.delete()
        """
        import labstep.entities.resource.repository as resourceRepository

        return resourceRepository.editResource(self, deleted_at=getTime())

    def addMetadata(
        self,
        fieldName,
        fieldType="default",
        value=None,
        date=None,
        number=None,
        unit=None,
        filepath=None,
        extraParams={},
    ):
        """
        Add Metadata to a Resource.

        Parameters
        ----------
        fieldName (str)
            The name of the field.
        fieldType (str)
            The Metadata field type. Options are: "default", "date",
            "numeric", or "file". The "default" type is "Text".
        value (str)
            The value accompanying the fieldName entry.
        date (str)
            The date accompanying the fieldName entry. Must be
            in the format of "YYYY-MM-DD HH:MM".
        number (float)
            The numeric value.
        unit (str)
            The unit accompanying the number entry.
        filepath (str)
            Local path to the file to upload for type 'file'

        Returns
        -------
        :class:`~labstep.entities.metadata.model.Metadata`
            An object representing the new Labstep Metadata.

        Example
        -------
        ::

            resource = user.getResource(17000)
            metadata = resource.addMetadata("Refractive Index",
                                               value="1.73")
        """
        import labstep.entities.metadata.repository as metadataRepository

        return metadataRepository.addMetadataTo(
            self,
            fieldName=fieldName,
            fieldType=fieldType,
            value=value,
            date=date,
            number=number,
            unit=unit,
            filepath=filepath,
            extraParams=extraParams,
        )

    def getMetadata(self):
        """
        Retrieve the Metadata of a Labstep Resource.

        Returns
        -------
        :class:`~labstep.entities.metadata.model.Metadata`
            An array of Metadata objects for the Resource.

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            metadatas = my_resource.getMetadata()
            metadatas[0].attributes()
        """
        import labstep.entities.metadata.repository as metadataRepository

        return metadataRepository.getMetadata(self)

    def getResourceCategory(self):
        """
        Get the ResourceCategory of the Resource.

        Returns
        -------
        :class:`~labstep.entities.resourceCategory.model.ResourceCategory`
            An object representing the Resource Category on Labstep.

        Example
        -------
        ::

            # Get a Resource
            resource = user.getResource(170)

            # Get the Resource Category of the resource
            resourceCategory = resource.getResourceCategory()
        """
        import labstep.entities.resourceCategory.repository as resourceCategoryRepository

        return resourceCategoryRepository.getResourceCategory(
            self.__user__, self.template["id"]
        )

    def setResourceCategory(self, resource_category_id, extraParams={}):
        """
        Add a Labstep ResourceCategory to a Resource.

        Parameters
        ----------
        resource_category_id (int)
            The id of :class:`~labstep.entities.resourceCategory.model.ResourceCategory`
            to set for
            the Resource.

        Returns
        -------
        :class:`~labstep.entities.resource.model.Resource`
            An object representing the Resource on Labstep.

        Example
        -------
        ::

            # Get a ResourceCategory
            resource_category = user.getResourceCategory(170)

            # Set the Resource Category
            my_resource = my_resource.setResourceCategory(resource_category.id)
        """
        import labstep.entities.resource.repository as resourceRepository

        return resourceRepository.editResource(
            self, resource_category_id=resource_category_id, extraParams=extraParams
        )

    def newOrderRequest(self, quantity=1, extraParams={}):
        """
        Create a new Labstep OrderRequest.

        Parameters
        ----------
        quantity (int)
            The quantity of the new OrderRequest.

        Returns
        -------
        :class:`~labstep.entities.orderRequest.model.OrderRequest`
            An object representing the new OrderRequest on Labstep.

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            order_request = my_resource.newOrderRequest(quantity=2)
        """
        import labstep.entities.orderRequest.repository as orderRequestRepository

        return orderRequestRepository.newOrderRequest(
            self.__user__,
            resource_id=self.id,
            quantity=quantity,
            extraParams=extraParams,
        )

    def getItems(self, count=100, search_query=None, extraParams={}):
        """
        Returns the items of this Resource.

        Parameters
        ----------
        count (int)
            The number of ResourceItems to retrieve.
        search_query (str)
            Search for ResourceItems with this 'name'.
        extraParams (dict)
            Dictionary of extra filter parameters.

        Returns
        -------
        List[:class:`~labstep.entities.resourceItem.model.ResourceItem`]
            A list of ResourceItem objects.

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            items = my_resource.getItems()
        """
        import labstep.entities.resourceItem.repository as resourceItemRepository

        return resourceItemRepository.getResourceItems(
            self.__user__,
            resource_id=self.id,
            count=count,
            search_query=search_query,
            extraParams=extraParams,
        )

    def newItem(
        self,
        name=None,
        availability="available",
        quantity_amount=None,
        quantity_unit=None,
        resource_location_id=None,
        extraParams={},
    ):
        """
        Create a new Labstep ResourceItem.

        Parameters
        ----------
        name (str)
            The new name of the ResourceItem.
        availability (str)
            The status of the ResourceItem. Options are:
            "available" and "unavailable".
        quantity_amount (float)
            The quantity of the ResourceItem.
        quantity_unit (str)
            The unit of the quantity.
        resource_location_id (int)
            The id of the ResourceLocation of the ResourceItem.

        Returns
        -------
        :class:`~labstep.entities.resourceItem.model.ResourceItem`
            An object representing a ResourceItem on Labstep.

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            item = my_resource.newItem(name='Test Item')
        """
        import labstep.entities.resourceItem.repository as resourceItemRepository

        return resourceItemRepository.newResourceItem(
            self.__user__,
            name=name,
            resource_id=self.id,
            availability=availability,
            quantity_amount=quantity_amount,
            quantity_unit=quantity_unit,
            resource_location_id=resource_location_id,
            extraParams=extraParams,
        )

    def getItemTemplate(self):
        """
        Get the template used for initialising new items of the resource.

        Returns
        -------
        :class:`~labstep.entities.resourceItem.model.ResourceItem`
            An object representing a ResourceItem on Labstep.

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            itemTemplate = my_resource.getItemTemplate()

            itemTemplate.addMetadata('Expiry Date')

        """
        self.update()
        if self.resource_item_template is None:
            return None

        return ResourceItem(self.resource_item_template, self.__user__)

    def enableCustomItemTemplate(self):
        """
        Enable a custom item template for this resource.
        If disabled the template for the Resource Category will be used.

        Returns
        -------
        None

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            my_resource.enableCustomItemTemplate()

            itemTemplate = my_resource.getItemTemplate()

            itemTemplate.addMetadata('Expiry Date')

        """
        if self.resource_item_template is None:
            self.newItem(extraParams={"is_template": 1})
        else:
            self.getItemTemplate().edit(extraParams={"deleted_at": None})

    def disableCustomItemTemplate(self):
        """
        Disable custom item template for this resource.
        If disabled the template for the Resource Category will be used.

        Returns
        -------
        None

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            my_resource.disableCustomItemTemplate()
        """
        self.getItemTemplate().delete()

    def getData(self):
        """
        Returns data linked to this resource via experiments.

        Returns
        -------
        List[:class:`~labstep.entities.experimentDataField.model.ExperimentDataField`]

        Example
        -------
        ::

            my_resource = user.getResource(17000)
            my_resource.getData()
        """
        import labstep.entities.experimentDataField.repository as experimentDataFieldRepository

        return experimentDataFieldRepository.getDataFields(self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity
from labstep.service.helpers import getTime


class ResourceLocation(Entity):
    """
    Represents a Resource Location on Labstep.

    To see all attributes of the resource location run
    ::
        print(my_resource_location)

    Specific attributes can be accessed via dot notation like so...
    ::
        print(my_resource_location.name)
        print(my_resource_location.id)
    """

    __entityName__ = "resource-location"
    __hasParentGroup__ = True

    def edit(self, name=None, extraParams={}):
        """
        Edit an existing ResourceLocation.

        Parameters
        ----------
        name (str)
            The new name of the ResourceLocation.

        Returns
        -------
        :class:`~labstep.entities.resourceLocation.model.ResourceLocation`
            An object representing the edited ResourceLocation.

        Example
        -------
        ::

            resource_location = user.getResourceLocation(123)
            resource_location.edit(name='A New ResourceLocation Name')
        """
        import labstep.entities.resourceLocation.repository as resourceLocationRepository

        return resourceLocationRepository.editResourceLocation(
            self, name, extraParams=extraParams
        )

    def delete(self):
        """
        Delete an existing ResourceLocation.

        Returns
        -------
        :class:`~labstep.entities.resourceLocation.model.ResourceLocation`
            An object representing the deleted ResourceLocation.

        Example
        -------
        ::

            resource_location = user.getResourceLocation(123)
            resource_location.delete()
        """
        return self.edit(extraParams={"deleted_at": getTime()})

    def addComment(self, body, filepath=None, extraParams={}):
        """
        Add a comment and/or file to a Labstep ResourceLocation.

        Parameters
        ----------
        body (str)
            The body of the comment.
        filepath (str)
            Filepath of file to attach to the comment

        Returns
        -------
        :class:`~labstep.entities.comment.model.Comment`
            The comment added.

        Example
        -------
        ::

            resource_location = user.getResourceLocation(123)
            resource_location.edit(name='A New ResourceLocation Name')
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.addCommentWithFile(
            self, body, filepath, extraParams=extraParams
        )

    def getComments(self, count=100, extraParams={}):
        """
        Retrieve the Comments attached to this Labstep Entity.

        Returns
        -------
        List[:class:`~labstep.entities.comment.model.Comment`]
            List of the comments attached.

        Example
        -------
        ::

            entity = user.getResource(17000)
            item = entity.getItems()[1]
            comments = item.getComments()
            comments[0].attributes()
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.getComments(self, count, extraParams=extraParams)

    def addMetadata(
        self,
        fieldName,
        fieldType="default",
        value=None,
        date=None,
        number=None,
        unit=None,
        filepath=None,
        extraParams={},
    ):
        """
        Add Metadata to a ResourceLocation.

        Parameters
        ----------
        fieldName (str)
            The name of the field.
        fieldType (str)
            The Metadata field type. Options are: "default", "date",
            "numeric", or "file". The "default" type is "Text".
        value (str)
            The value accompanying the fieldName entry.
        date (str)
            The date accompanying the fieldName entry. Must be
            in the format of "YYYY-MM-DD HH:MM".
        number (float)
            The numeric value.
        unit (str)
            The unit accompanying the number entry.
        filepath (str)
            Local path to the file to upload for type 'file'

        Returns
        -------
        :class:`~labstep.entities.metadata.model.Metadata`
            An object representing the new Labstep Metadata.

        Example
        -------
        ::

            resource_location = user.getResourceLocation(123)
            resource_location.addMetadata("Refractive Index",
                                                    value="1.73")
        """
        import labstep.entities.metadata.repository as metadataRepository

        return metadataRepository.addMetadataTo(
            self,
            fieldName,
            fieldType=fieldType,
            value=value,
            date=date,
            number=number,
            unit=unit,
            filepath=filepath,
            extraParams=extraParams,
        )

    def getMetadata(self):
        """
        Retrieve the Metadata of a Labstep ResourceLocation.

        Returns
        -------
        List[:class:`~labstep.entities.metadata.model.Metadata`]
            An array of Metadata objects for the ResourceLocation.

        Example
        -------
        ::

            resource_location = user.getResourceLocation(123)
            metadata = resource_location.getMetadata()
        """
        import labstep.entities.metadata.repository as metadataRepository

        return metadataRepository.getMetadata(self)

    def getItems(self, count=100, extraParams={}):
        """
        Get a list of items in this location.

        Returns
        -------
        List[:class:`~labstep.entities.resourceItem.model.ResourceItem`]
            An array of items in the ResourceLocation.

        Example
        -------
        ::

            resource_location = user.getResourceLocation(123)
            items = resource_location.getItems() 
        """
        import labstep.entities.resourceItem.repository as resourceItemRepository
        return resourceItemRepository.getResourceItems(self.__user__, count=count, extraParams={'resource_location_id': self.id, **extraParams})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class ExperimentMaterial(Entity):
    __entityName__ = "experiment-value"

    def __init__(self, data, user):
        super().__init__(data, user)
        self.amount = self.value

    def edit(self, name=None, amount=None, units=None, resource_id=None, resource_item_id=None, extraParams={}):
        """
        Edit an existing Experiment Material.

        Parameters
        ----------
        amount (str)
            The amount of the Experiment Material.
        units (str)
            The units of the amount.
        resource_id (int)
            The :class:`~labstep.entities.resource.model.Resource` of the Experiment Material.
        resource_item_id (int)
            The id of the :class:`~labstep.entities.resource.model.ResourceItem`
            of the Experiment Material.

        Returns
        -------
        :class:`~labstep.entities.experimentMaterial.model.ExperimentMaterial`
            An object representing the edited Experiment Material.

        Example
        -------
        ::

            experiment = user.getExperiment(17000)
            exp_protocol = experiment.getProtocols()[0]
            exp_protocol_materials = exp_protocol.getMaterials()
            exp_protocol_materials[0].edit(amount=1.7, units='ml')
        """
        import labstep.entities.experimentMaterial.repository as experimentMaterialRepository

        return experimentMaterialRepository.editExperimentMaterial(self,
                                                                   name=name,
                                                                   amount=amount,
                                                                   units=units,
                                                                   resource_id=resource_id,
                                                                   resource_item_id=resource_item_id,
                                                                   extraParams=extraParams)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.entities.workspace.model import Workspace
import labstep.generic.entity.repository as entityRepository


def getWorkspace(user, workspace_id):
    return entityRepository.getEntity(user, Workspace, id=workspace_id)


def getWorkspaces(user, count=100, search_query=None, extraParams={}):
    params = {"name": search_query, **extraParams}
    return entityRepository.getEntities(user, Workspace, count, params)


def newWorkspace(user, name, extraParams={}):
    params = {"name": name, **extraParams}
    return entityRepository.newEntity(user, Workspace, params)


def editWorkspace(workspace, name=None, deleted_at=None, extraParams={}):
    params = {"name": name, "deleted_at": deleted_at, **extraParams}
    return entityRepository.editEntity(workspace, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class ExperimentTable(Entity):
    __entityName__ = "experiment-table"

    def edit(self, name=None, data=None):
        """
        Edit an existing Experiment Table.

        Parameters
        ----------
        name (str)
            The name of the Experiment Table.
        data (str)
            The data of the table.

        Returns
        -------
        :class:`~labstep.entities.experimentTable.model.ExperimentTable`
            An object representing the edited Experiment Table.

        Example
        -------
        ::

            experiment = user.getExperiment(17000)
            exp_protocol = experiment.getProtocols()[0]
            exp_protocol_tables = exp_protocol.getTables()
            data = {
                "rowCount": 12,
                "columnCount": 12,
                "colHeaderData": {},
                "data": {
                    "dataTable": {
                        0: {
                            0: {
                                "value": 'Cell A1'
                            },
                            1: {
                                "value": 'Cell B1'
                            }
                        }
                    }
                }
            }
            exp_protocol_tables[0].edit(data=data)
        """
        import labstep.generic.entity.repository as entityRepository

        params = {"name": name, "data": data}
        return entityRepository.editEntity(self, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class ShareableEntity(Entity):
    __hasParentGroup__ = True

    def addComment(self, body, filepath=None, extraParams={}):
        """
        Add a comment and/or file to a Labstep Entity.

        Parameters
        ----------
        body (str)
            The body of the comment.
        filepath (str)
            A Labstep File entity to attach to the comment,
            including the filepath.

        Returns
        -------
        :class:`~labstep.entities.comment.model.Comment`
            The comment added.

        Example
        -------
        ::

            my_experiment = user.getExperiment(17000)
            my_experiment.addComment(body='I am commenting!',
                                     filepath='pwd/file_to_upload.dat')
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.addCommentWithFile(
            self, body=body, filepath=filepath, extraParams=extraParams
        )

    def getComments(self, count=100):
        """
        Retrieve the Comments attached to this Labstep Entity.

        Returns
        -------
        List[:class:`~labstep.entities.comment.model.Comment`]
            List of the comments attached.

        Example
        -------
        ::

            entity = user.getExperiment(17000)
            comments = entity.getComments()
            comments[0].attributes()
        """
        import labstep.entities.comment.repository as commentRepository

        return commentRepository.getComments(self, count)

    def getPermissions(self):
        """
        Returns the sharing permissions for the Entity.

        Returns
        -------
        List[:class:`~labstep.entities.permission.model.Permission`]
        """
        import labstep.entities.permission.repository as permissionRepository

        return permissionRepository.getPermissions(self)

    def getSharelink(self):
        """
        Returns a sharelink for the Entity.

        Returns
        -------
        :class:`~labstep.entities.sharelink.model.Sharelink`
            The sharelink for the entity
        """
        import labstep.entities.sharelink.repository as shareLinkRepository

        return shareLinkRepository.getSharelink(self)

    def shareWith(self, workspace_id, permission="view"):
        """
        Shares the Entity with another Workspace.

        Parameters
        ----------
        workspace_id (int)
            The id of the workspace to share with

        permission (str)
            Permission to share with. Can be 'view' or 'edit'

        Returns
        -------
        None
        """
        import labstep.entities.permission.repository as permissionRepository

        return permissionRepository.newPermission(self, workspace_id, permission)

    def transferOwnership(self, workspace_id):
        """
        Transfer ownership of the Entity to a different Workspace

        Parameters
        ----------
        workspace_id (int)
            The id of the workspace to transfer ownership to
        """
        import labstep.entities.permission.repository as permissionRepository

        return permissionRepository.transferOwnership(self, workspace_id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.generic.entity.model import Entity


class ExperimentTimer(Entity):
    __entityName__ = "experiment-timer"

    def edit(self, hours=None, minutes=None, seconds=None):
        """
        Edit an existing Experiment Timer.

        Parameters
        ----------
        hours (int)
            The hours of the timer.
        minutes (int)
            The minutes of the timer.
        seconds (int)
            The seconds of the timer.

        Returns
        -------
        :class:`~labstep.entities.experimentTimer.model.ExperimentTimer`
            An object representing the edited Experiment Timer.

        Example
        -------
        ::

            experiment = user.getExperiment(17000)
            exp_protocol = experiment.getProtocols()[0]
            exp_protocol_timers = exp_protocol.getTimers()
            exp_protocol_timers[0].edit(minutes=1, seconds=7)
        """
        import labstep.generic.entity.repository as entityRepository

        fields = {}

        if hours is not None:
            fields["hours"] = hours
        if minutes is not None:
            fields["minutes"] = minutes
        if seconds is not None:
            fields["seconds"] = seconds

        return entityRepository.editEntity(self, fields)

    """ def start(self):
        time = getTime() + self.hours + self.minutes + self.seconds
        fields = {'ended_at': time}
        return editEntity(self, fields)

    def pause(self):
        time = getTime()
        fields = {'paused_at': time}
        return editEntity(self, fields)

    def resume(self):
        time = getTime() + self.ended_at - self.paused_at
        fields = {'ended_at': time
                  'paused_at': 'null',
                  }
        return editEntity(self, fields) """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.entities.device.model import Device
import labstep.generic.entity.repository as entityRepository


def getDevice(user, device_id):
    return entityRepository.getEntity(user, Device, id=device_id)


def getDevices(user, count=100, search_query=None, extraParams={}):
    params = {"search_query": search_query, **extraParams}
    return entityRepository.getEntities(user, Device, count, params)


def newDevice(user, name, extraParams={}):
    params = {"name": name, **extraParams}
    return entityRepository.newEntity(user, Device, params)


def editDevice(device, name=None, deleted_at=None, extraParams={}):
    params = {"name": name, "deleted_at": deleted_at, **extraParams}
    return entityRepository.editEntity(device, params)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

from labstep.service.htmlToProseMirror import htmlToProseMirrorService


def parseHTML(authenticatedUser, html):
    """
    Converts HTML into the JSON format used in protocols
    and experiment entries.

    Parameters
    ----------
    authenticatedUser (:)
        An authenticated Labstep :class:`~labstep.entities.user.model.User`
        (see :func:`~labstep.authenticate`)
    html (str)
        HTML to be converted (as a string).
    """
    return htmlToProseMirrorService.parseHTML(authenticatedUser, html)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Author: Barney Walker <barney@labstep.com>

import labstep.entities.file.repository as fileRepository
import labstep.generic.entity.repository as entityRepository
from labstep.service.helpers import handleDate, handleString
from labstep.entities.protocolDataField.model import ProtocolDataField
from labstep.entities.protocolVersion.model import ProtocolVersion


def getDataFields(entity, count=1000, extraParams={}):

    if isinstance(entity, ProtocolVersion):
        params = {
            "metadata_thread_id": entity.metadata_thread["id"]}

        return entityRepository.getEntities(
            entity.__user__, ProtocolDataField, count=count, filterParams={
                **params, **extraParams}
        )


def addDataFieldTo(

    entity,
    fieldName,
    fieldType="default",
    value=None,
    date=None,
    number=None,
    unit=None,
    filepath=None,
    extraParams={},
):
    if filepath is not None:
        fileId = fileRepository.newFile(entity.__user__, filepath).id
    else:
        fileId = None

    params = {
        "metadata_thread_id": entity.metadata_thread["id"],
        "type": fieldType,
        "label": handleString(fieldName),
        "value": handleString(value),
        "date": handleDate(date),
        "number": number,
        "unit": unit,
        "file_id": fileId,
        **extraParams,
    }

    dataField = entityRepository.newEntity(
        entity.__user__, ProtocolDataField, params)

    dataField.protocol_id = entity.id

    return dataField


def editDataField(dataField, fieldName=None, value=None, extraParams={}):
    params = {
        "label": handleString(fieldName),
        "value": handleString(value),
        **extraParams
    }
    return entityRepository.editEntity(dataField, params)
